@@ -67,7 +67,7 @@ ags_pattern_envelope_get_type(void)
   static volatile gsize g_define_type_id__volatile = 0;
 
   if(g_once_init_enter (&g_define_type_id__volatile)){
-    GType ags_type_pattern_envelope;
+    GType ags_type_pattern_envelope = 0;
 
     static const GTypeInfo ags_pattern_envelope_info = {
       sizeof (AgsPatternEnvelopeClass),
@@ -105,7 +105,7 @@ ags_pattern_envelope_get_type(void)
 				AGS_TYPE_APPLICABLE,
 				&ags_applicable_interface_info);
 
-    g_once_init_leave (&g_define_type_id__volatile, ags_type_pattern_envelope);
+    g_once_init_leave(&g_define_type_id__volatile, ags_type_pattern_envelope);
   }
 
   return g_define_type_id__volatile;
@@ -973,7 +973,7 @@ ags_pattern_envelope_y_label_func(gdouble value,
  * 
  * Returns: the matching #AgsPreset, if none selected %NULL
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsPreset*
 ags_pattern_envelope_get_active_preset(AgsPatternEnvelope *pattern_envelope)
@@ -988,20 +988,15 @@ ags_pattern_envelope_get_active_preset(AgsPatternEnvelope *pattern_envelope)
 
   AgsAudio *audio;
   AgsPreset *current;
-  
-  AgsMutexManager *mutex_manager;
-  
+    
   AgsApplicationContext *application_context;
 
-  GList *preset;
+  GList *start_preset, *preset;
 
   gchar *preset_name;
   
   gboolean do_edit;
 
-  pthread_mutex_t *application_mutex;
-  pthread_mutex_t *audio_mutex;
-
   if(!AGS_IS_PATTERN_ENVELOPE(pattern_envelope)){
     return(NULL);
   }
@@ -1018,17 +1013,6 @@ ags_pattern_envelope_get_active_preset(AgsPatternEnvelope *pattern_envelope)
   /* application context and mutex manager */
   application_context = window->application_context;
 
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-
-  /* get audio mutex */
-  pthread_mutex_lock(application_mutex);
-
-  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
-					 (GObject *) audio);
-  
-  pthread_mutex_unlock(application_mutex);
-
   /* get model */
   model = gtk_tree_view_get_model(pattern_envelope->tree_view);
 
@@ -1057,9 +1041,11 @@ ags_pattern_envelope_get_active_preset(AgsPatternEnvelope *pattern_envelope)
 		     -1);
 
   /* find preset */
-  pthread_mutex_lock(audio_mutex);
+  g_object_get(audio,
+	       "preset", &start_preset,
+	       NULL);
 
-  preset = audio->preset;
+  preset = start_preset;
   current = NULL;
   
   preset = ags_preset_find_name(preset,
@@ -1069,8 +1055,6 @@ ags_pattern_envelope_get_active_preset(AgsPatternEnvelope *pattern_envelope)
     current = preset->data;
   }
 
-  pthread_mutex_unlock(audio_mutex);
-
   return(current);
 }
 
@@ -1080,7 +1064,7 @@ ags_pattern_envelope_get_active_preset(AgsPatternEnvelope *pattern_envelope)
  *
  * Load preset.
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_pattern_envelope_load_preset(AgsPatternEnvelope *pattern_envelope)
@@ -1094,15 +1078,10 @@ ags_pattern_envelope_load_preset(AgsPatternEnvelope *pattern_envelope)
   GtkTreeIter iter;
 
   AgsAudio *audio;
-
-  AgsMutexManager *mutex_manager;
   
   AgsApplicationContext *application_context;
   
-  GList *preset;
-
-  pthread_mutex_t *application_mutex;
-  pthread_mutex_t *audio_mutex;
+  GList *start_preset, *preset;
   
   if(!AGS_IS_PATTERN_ENVELOPE(pattern_envelope)){
     return;
@@ -1119,17 +1098,6 @@ ags_pattern_envelope_load_preset(AgsPatternEnvelope *pattern_envelope)
 
   /* application context and mutex manager */
   application_context = window->application_context;
-
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-
-  /* get audio mutex */
-  pthread_mutex_lock(application_mutex);
-
-  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
-					 (GObject *) audio);
-  
-  pthread_mutex_unlock(application_mutex);
   
   /* get model */
   model = GTK_LIST_STORE(gtk_tree_view_get_model(pattern_envelope->tree_view));
@@ -1138,9 +1106,11 @@ ags_pattern_envelope_load_preset(AgsPatternEnvelope *pattern_envelope)
   gtk_list_store_clear(model);
 
   /* create new */
-  pthread_mutex_lock(audio_mutex);
+  g_object_get(audio,
+	       "preset", &start_preset,
+	       NULL);
 
-  preset = audio->preset;
+  preset = start_preset;
 
   while(preset != NULL){
     gtk_list_store_append(model,
@@ -1157,8 +1127,6 @@ ags_pattern_envelope_load_preset(AgsPatternEnvelope *pattern_envelope)
 
     preset = preset->next;
   }
-
-  pthread_mutex_unlock(audio_mutex);
 }
 
 /**
@@ -1170,7 +1138,7 @@ ags_pattern_envelope_load_preset(AgsPatternEnvelope *pattern_envelope)
  * 
  * Set preset property.
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_pattern_envelope_set_preset_property(AgsPatternEnvelope *pattern_envelope,
@@ -1184,13 +1152,10 @@ ags_pattern_envelope_set_preset_property(AgsPatternEnvelope *pattern_envelope,
 
   AgsAudio *audio;
 
-  AgsMutexManager *mutex_manager;
+  GList *start_preset;
   
   AgsApplicationContext *application_context;
-  
-  pthread_mutex_t *application_mutex;
-  pthread_mutex_t *audio_mutex;
-  
+    
   if(!AGS_IS_PATTERN_ENVELOPE(pattern_envelope)){
     return;
   }
@@ -1207,24 +1172,13 @@ ags_pattern_envelope_set_preset_property(AgsPatternEnvelope *pattern_envelope,
   /* application context and mutex manager */
   application_context = window->application_context;
 
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-
-  /* get audio mutex */
-  pthread_mutex_lock(application_mutex);
-
-  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
-					 (GObject *) audio);
-  
-  pthread_mutex_unlock(application_mutex);
-
   /* set property */  
-  pthread_mutex_lock(audio_mutex);
+  g_object_get(audio,
+	       "preset", &start_preset,
+	       NULL);
 
-  g_object_set_property(preset,
+  g_object_set_property(start_preset,
 			property_name, value);
-
-  pthread_mutex_unlock(audio_mutex);
   
   /* reset */
   ags_pattern_envelope_reset_tree_view(pattern_envelope);
@@ -1237,7 +1191,7 @@ ags_pattern_envelope_set_preset_property(AgsPatternEnvelope *pattern_envelope,
  *
  * Add preset.
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_pattern_envelope_add_preset(AgsPatternEnvelope *pattern_envelope,
@@ -1250,17 +1204,14 @@ ags_pattern_envelope_add_preset(AgsPatternEnvelope *pattern_envelope,
 
   AgsAudio *audio;
   AgsPreset *preset;
-
-  AgsMutexManager *mutex_manager;
   
   AgsApplicationContext *application_context;
 
+  GList *start_preset;
+  
   AgsComplex *val;
   
   GValue value = {0,};
-
-  pthread_mutex_t *application_mutex;
-  pthread_mutex_t *audio_mutex;
   
   if(!AGS_IS_PATTERN_ENVELOPE(pattern_envelope) ||
      preset_name == NULL){
@@ -1279,24 +1230,13 @@ ags_pattern_envelope_add_preset(AgsPatternEnvelope *pattern_envelope,
   /* application context and mutex manager */
   application_context = window->application_context;
 
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-
-  /* get audio mutex */
-  pthread_mutex_lock(application_mutex);
-
-  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
-					 (GObject *) audio);
-  
-  pthread_mutex_unlock(application_mutex);
-
   /* check if already present */
-  pthread_mutex_lock(audio_mutex);
+  g_object_get(audio,
+	       "preset", &start_preset,
+	       NULL);
 
-  if(ags_preset_find_name(audio->preset,
+  if(ags_preset_find_name(start_preset,
 			  preset_name) != NULL){
-    pthread_mutex_unlock(audio_mutex);
-    
     return;
   }
   
@@ -1370,7 +1310,6 @@ ags_pattern_envelope_add_preset(AgsPatternEnvelope *pattern_envelope,
 			   "release", &value);
 
   /* release mutex */
-  pthread_mutex_unlock(audio_mutex);
 }
 
 /**
@@ -1380,7 +1319,7 @@ ags_pattern_envelope_add_preset(AgsPatternEnvelope *pattern_envelope,
  *
  * Remove preset.
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_pattern_envelope_remove_preset(AgsPatternEnvelope *pattern_envelope,
@@ -1393,13 +1332,10 @@ ags_pattern_envelope_remove_preset(AgsPatternEnvelope *pattern_envelope,
 
   AgsAudio *audio;
   AgsPreset *preset;
-
-  AgsMutexManager *mutex_manager;
   
   AgsApplicationContext *application_context;
 
-  pthread_mutex_t *application_mutex;
-  pthread_mutex_t *audio_mutex;
+  GList *start_preset;
   
   if(!AGS_IS_PATTERN_ENVELOPE(pattern_envelope)){
     return;
@@ -1417,24 +1353,13 @@ ags_pattern_envelope_remove_preset(AgsPatternEnvelope *pattern_envelope,
   /* application context and mutex manager */
   application_context = window->application_context;
 
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-
-  /* get audio mutex */
-  pthread_mutex_lock(application_mutex);
-
-  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
-					 (GObject *) audio);
-  
-  pthread_mutex_unlock(application_mutex);
-
   /* create preset */
-  pthread_mutex_lock(audio_mutex);
+  g_object_get(audio,
+	       "preset", &start_preset,
+	       NULL);
 
-  preset = g_list_nth_data(audio->preset,
+  preset = g_list_nth_data(start_preset,
 			   nth);
-  
-  pthread_mutex_unlock(audio_mutex);
 
   ags_audio_remove_preset(audio,
 			  preset);
@@ -1446,7 +1371,7 @@ ags_pattern_envelope_remove_preset(AgsPatternEnvelope *pattern_envelope,
  * 
  * Reset controls.
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_pattern_envelope_reset_control(AgsPatternEnvelope *pattern_envelope)
@@ -1458,8 +1383,6 @@ ags_pattern_envelope_reset_control(AgsPatternEnvelope *pattern_envelope)
 
   AgsAudio *audio;
   AgsPreset *preset;
-
-  AgsMutexManager *mutex_manager;
   
   AgsApplicationContext *application_context;
 
@@ -1472,9 +1395,6 @@ ags_pattern_envelope_reset_control(AgsPatternEnvelope *pattern_envelope)
   GValue value = {0,};
 
   GError *error;
-
-  pthread_mutex_t *application_mutex;
-  pthread_mutex_t *audio_mutex;
   
   if(!AGS_IS_PATTERN_ENVELOPE(pattern_envelope)){
     return;
@@ -1504,20 +1424,7 @@ ags_pattern_envelope_reset_control(AgsPatternEnvelope *pattern_envelope)
   /* application context and mutex manager */
   application_context = window->application_context;
 
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-
-  /* get audio mutex */
-  pthread_mutex_lock(application_mutex);
-
-  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
-					 (GObject *) audio);
-  
-  pthread_mutex_unlock(application_mutex);
-
   /* get preset properties  */
-  pthread_mutex_lock(audio_mutex);
-
   g_object_get(preset,
 	       "audio-channel-start", &audio_channel_start,
 	       "audio-channel-end", &audio_channel_end,
@@ -1556,8 +1463,6 @@ ags_pattern_envelope_reset_control(AgsPatternEnvelope *pattern_envelope)
 
     pattern_envelope->flags &= (~AGS_PATTERN_ENVELOPE_NO_UPDATE);
     
-    pthread_mutex_unlock(audio_mutex);
-
     return;
   }
 
@@ -1581,8 +1486,6 @@ ags_pattern_envelope_reset_control(AgsPatternEnvelope *pattern_envelope)
 
     pattern_envelope->flags &= (~AGS_PATTERN_ENVELOPE_NO_UPDATE);
 
-    pthread_mutex_unlock(audio_mutex);
-
     return;
   }
 
@@ -1606,8 +1509,6 @@ ags_pattern_envelope_reset_control(AgsPatternEnvelope *pattern_envelope)
 
     pattern_envelope->flags &= (~AGS_PATTERN_ENVELOPE_NO_UPDATE);
 
-    pthread_mutex_unlock(audio_mutex);
-
     return;
   }
 
@@ -1631,8 +1532,6 @@ ags_pattern_envelope_reset_control(AgsPatternEnvelope *pattern_envelope)
 
     pattern_envelope->flags &= (~AGS_PATTERN_ENVELOPE_NO_UPDATE);
 
-    pthread_mutex_unlock(audio_mutex);
-
     return;
   }
 
@@ -1656,8 +1555,6 @@ ags_pattern_envelope_reset_control(AgsPatternEnvelope *pattern_envelope)
 
     pattern_envelope->flags &= (~AGS_PATTERN_ENVELOPE_NO_UPDATE);
 
-    pthread_mutex_unlock(audio_mutex);
-
     return;
   }
 
@@ -1668,9 +1565,6 @@ ags_pattern_envelope_reset_control(AgsPatternEnvelope *pattern_envelope)
 
   /* unset no update */
   pattern_envelope->flags &= (~AGS_PATTERN_ENVELOPE_NO_UPDATE);
-
-  /* release mutex */
-  pthread_mutex_unlock(audio_mutex);
 }
 
 /**
@@ -1679,7 +1573,7 @@ ags_pattern_envelope_reset_control(AgsPatternEnvelope *pattern_envelope)
  * 
  * Reset tree view.
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_pattern_envelope_reset_tree_view(AgsPatternEnvelope *pattern_envelope)
@@ -1694,16 +1588,11 @@ ags_pattern_envelope_reset_tree_view(AgsPatternEnvelope *pattern_envelope)
 
   AgsAudio *audio;
   AgsPreset *preset;
-
-  AgsMutexManager *mutex_manager;
   
   AgsApplicationContext *application_context;
 
   gboolean do_edit;
 
-  pthread_mutex_t *application_mutex;
-  pthread_mutex_t *audio_mutex;
-
   if(!AGS_IS_PATTERN_ENVELOPE(pattern_envelope)){
     return;
   }
@@ -1732,17 +1621,6 @@ ags_pattern_envelope_reset_tree_view(AgsPatternEnvelope *pattern_envelope)
   /* application context and mutex manager */
   application_context = window->application_context;
 
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-
-  /* get audio mutex */
-  pthread_mutex_lock(application_mutex);
-
-  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
-					 (GObject *) audio);
-  
-  pthread_mutex_unlock(application_mutex);
-
   /* get model and get editing iter */
   model = gtk_tree_view_get_model(pattern_envelope->tree_view);
 
@@ -1765,9 +1643,7 @@ ags_pattern_envelope_reset_tree_view(AgsPatternEnvelope *pattern_envelope)
     guint audio_channel_start, audio_channel_end;
     guint pad_start, pad_end;
     guint x_start, x_end;
-    
-    pthread_mutex_lock(audio_mutex);
-    
+        
     g_object_get(preset,
 		 "audio-channel-start", &audio_channel_start,
 		 "audio-channel-end", &audio_channel_end,
@@ -1785,8 +1661,6 @@ ags_pattern_envelope_reset_tree_view(AgsPatternEnvelope *pattern_envelope)
 		       AGS_PATTERN_ENVELOPE_COLUMN_X_START, x_start,
 		       AGS_PATTERN_ENVELOPE_COLUMN_X_END, x_end,
 		       -1);
-
-    pthread_mutex_unlock(audio_mutex);
   }
   
   /* unset no update */
@@ -1799,7 +1673,7 @@ ags_pattern_envelope_reset_tree_view(AgsPatternEnvelope *pattern_envelope)
  * 
  * Plot envelope.
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_pattern_envelope_plot(AgsPatternEnvelope *pattern_envelope)
@@ -1816,8 +1690,6 @@ ags_pattern_envelope_plot(AgsPatternEnvelope *pattern_envelope)
   GtkTreeIter iter;
 
   AgsAudio *audio;
-
-  AgsMutexManager *mutex_manager;
   
   AgsApplicationContext *application_context;
 
@@ -1834,9 +1706,6 @@ ags_pattern_envelope_plot(AgsPatternEnvelope *pattern_envelope)
   GValue value = {0,};
 
   GError *error;
-
-  pthread_mutex_t *application_mutex;
-  pthread_mutex_t *audio_mutex;
   
   if(!AGS_IS_PATTERN_ENVELOPE(pattern_envelope)){
     return;
@@ -1854,17 +1723,6 @@ ags_pattern_envelope_plot(AgsPatternEnvelope *pattern_envelope)
   /* application context and mutex manager */
   application_context = window->application_context;
 
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-
-  /* get audio mutex */
-  pthread_mutex_lock(application_mutex);
-
-  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
-					 (GObject *) audio);
-  
-  pthread_mutex_unlock(application_mutex);
-
   /* get dimension of cartesian */
   cartesian = pattern_envelope->cartesian;
 
@@ -1884,9 +1742,7 @@ ags_pattern_envelope_plot(AgsPatternEnvelope *pattern_envelope)
 
   /* plot */
   if(gtk_tree_model_get_iter_first(model,
-				   &iter)){
-    pthread_mutex_lock(audio_mutex);
-    
+				   &iter)){    
     g_value_init(&value,
 		 AGS_TYPE_COMPLEX);
     
@@ -2035,8 +1891,6 @@ ags_pattern_envelope_plot(AgsPatternEnvelope *pattern_envelope)
       }
     }while(gtk_tree_model_iter_next(model,
 				    &iter));
-
-    pthread_mutex_unlock(audio_mutex);
   }
 
   /* queue draw */
@@ -2050,7 +1904,7 @@ ags_pattern_envelope_plot(AgsPatternEnvelope *pattern_envelope)
  *
  * Returns: a new #AgsPatternEnvelope
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsPatternEnvelope*
 ags_pattern_envelope_new()
