@@ -24,6 +24,7 @@
 #include <ags/libags-audio.h>
 #include <ags/libags-gui.h>
 
+#include <ags/X/ags_ui_provider.h>
 #include <ags/X/ags_window.h>
 #include <ags/X/ags_machine.h>
 #include <ags/X/ags_pad.h>
@@ -42,8 +43,10 @@ void ags_syncsynth_connectable_interface_init(AgsConnectableInterface *connectab
 void ags_syncsynth_plugin_interface_init(AgsPluginInterface *plugin);
 void ags_syncsynth_init(AgsSyncsynth *syncsynth);
 void ags_syncsynth_finalize(GObject *gobject);
+
 void ags_syncsynth_connect(AgsConnectable *connectable);
 void ags_syncsynth_disconnect(AgsConnectable *connectable);
+
 void ags_syncsynth_show(GtkWidget *widget);
 void ags_syncsynth_map_recall(AgsMachine *machine);
 gchar* ags_syncsynth_get_name(AgsPlugin *plugin);
@@ -87,7 +90,7 @@ ags_syncsynth_get_type(void)
   static volatile gsize g_define_type_id__volatile = 0;
 
   if(g_once_init_enter (&g_define_type_id__volatile)){
-    GType ags_type_syncsynth;
+    GType ags_type_syncsynth = 0;
 
     static const GTypeInfo ags_syncsynth_info = {
       sizeof(AgsSyncsynthClass),
@@ -125,7 +128,7 @@ ags_syncsynth_get_type(void)
 				AGS_TYPE_PLUGIN,
 				&ags_plugin_interface_info);
 
-    g_once_init_leave (&g_define_type_id__volatile, ags_type_syncsynth);
+    g_once_init_leave(&g_define_type_id__volatile, ags_type_syncsynth);
   }
 
   return g_define_type_id__volatile;
@@ -186,14 +189,14 @@ ags_syncsynth_init(AgsSyncsynth *syncsynth)
 			 G_CALLBACK(ags_syncsynth_parent_set_callback), (gpointer) syncsynth);
 
   audio = AGS_MACHINE(syncsynth)->audio;
-  audio->flags |= (AGS_AUDIO_SYNC |
-		   AGS_AUDIO_ASYNC |
-		   AGS_AUDIO_OUTPUT_HAS_RECYCLING |
-		   AGS_AUDIO_INPUT_HAS_RECYCLING |
-		   AGS_AUDIO_INPUT_TAKES_SYNTH |
-		   AGS_AUDIO_HAS_NOTATION | 
-		   AGS_AUDIO_NOTATION_DEFAULT |
-		   AGS_AUDIO_REVERSE_MAPPING);
+  ags_audio_set_flags(audio, (AGS_AUDIO_SYNC |
+			      AGS_AUDIO_ASYNC |
+			      AGS_AUDIO_OUTPUT_HAS_RECYCLING |
+			      AGS_AUDIO_INPUT_HAS_RECYCLING |
+			      AGS_AUDIO_INPUT_HAS_SYNTH));
+  ags_audio_set_ability_flags(audio, (AGS_SOUND_ABILITY_NOTATION));
+  ags_audio_set_behaviour_flags(audio, (AGS_SOUND_BEHAVIOUR_REVERSE_MAPPING |
+					AGS_SOUND_BEHAVIOUR_DEFAULTS_TO_INPUT));
   g_object_set(audio,
 	       "audio-start-mapping", 0,
 	       "audio-end-mapping", 128,
@@ -476,7 +479,7 @@ ags_syncsynth_map_recall(AgsMachine *machine)
   AgsPlayNotationAudio *recall_notation_audio;
   AgsPlayNotationAudioRun *recall_notation_audio_run;
 
-  GList *list;
+  GList *start_play, *play;
 
   if((AGS_MACHINE_MAPPED_RECALL & (machine->flags)) != 0 ||
      (AGS_MACHINE_PREMAPPED_RECALL & (machine->flags)) != 0){
@@ -501,15 +504,22 @@ ags_syncsynth_map_recall(AgsMachine *machine)
 			     AGS_RECALL_FACTORY_PLAY),
 			    0);
 
-  list = ags_recall_find_type(audio->play, AGS_TYPE_DELAY_AUDIO_RUN);
+  g_object_get(audio,
+	       "play", &start_play,
+	       NULL);
+  
+  play = ags_recall_find_type(start_play,
+			      AGS_TYPE_DELAY_AUDIO_RUN);
 
-  if(list != NULL){
-    play_delay_audio_run = AGS_DELAY_AUDIO_RUN(list->data);
+  if(play != NULL){
+    play_delay_audio_run = AGS_DELAY_AUDIO_RUN(play->data);
     //    AGS_RECALL(play_delay_audio_run)->flags |= AGS_RECALL_PERSISTENT;
   }else{
     play_delay_audio_run = NULL;
   }
   
+  g_list_free(start_play);
+  
   /* ags-count-beats */
   ags_recall_factory_create(audio,
 			    NULL, NULL,
@@ -520,13 +530,18 @@ ags_syncsynth_map_recall(AgsMachine *machine)
 			     AGS_RECALL_FACTORY_ADD |
 			     AGS_RECALL_FACTORY_PLAY),
 			    0);
-  
-  list = ags_recall_find_type(audio->play, AGS_TYPE_COUNT_BEATS_AUDIO_RUN);
 
-  if(list != NULL){
+  g_object_get(audio,
+	       "play", &start_play,
+	       NULL);
+  
+  play = ags_recall_find_type(start_play,
+			      AGS_TYPE_COUNT_BEATS_AUDIO_RUN);
+
+  if(play != NULL){
     GValue value = {0,};
 
-    play_count_beats_audio_run = AGS_COUNT_BEATS_AUDIO_RUN(list->data);
+    play_count_beats_audio_run = AGS_COUNT_BEATS_AUDIO_RUN(play->data);
 
     /* set dependency */  
     g_object_set(G_OBJECT(play_count_beats_audio_run),
@@ -544,6 +559,8 @@ ags_syncsynth_map_recall(AgsMachine *machine)
     play_count_beats_audio_run = NULL;
   }
 
+  g_list_free(start_play);
+  
   /* ags-record-midi */
   ags_recall_factory_create(audio,
 			    NULL, NULL,
@@ -552,13 +569,18 @@ ags_syncsynth_map_recall(AgsMachine *machine)
 			    0, 0,
 			    (AGS_RECALL_FACTORY_INPUT |
 			     AGS_RECALL_FACTORY_ADD |
-			     AGS_RECALL_FACTORY_RECALL),
+			     AGS_RECALL_FACTORY_PLAY),
 			    0);
 
-  list = ags_recall_find_type(audio->recall, AGS_TYPE_RECORD_MIDI_AUDIO_RUN);
+  g_object_get(audio,
+	       "play", &start_play,
+	       NULL);
 
-  if(list != NULL){
-    recall_record_midi_audio_run = AGS_RECORD_MIDI_AUDIO_RUN(list->data);
+  play = ags_recall_find_type(start_play,
+			      AGS_TYPE_PLAY_NOTATION_AUDIO_RUN);
+
+  if(play != NULL){
+    recall_record_midi_audio_run = AGS_RECORD_MIDI_AUDIO_RUN(play->data);
     
     /* set dependency */
     g_object_set(G_OBJECT(recall_record_midi_audio_run),
@@ -571,6 +593,8 @@ ags_syncsynth_map_recall(AgsMachine *machine)
 		 NULL);
   }  
 
+  g_list_free(start_play);
+  
   /* ags-play-notation */
   ags_recall_factory_create(audio,
 			    NULL, NULL,
@@ -579,14 +603,19 @@ ags_syncsynth_map_recall(AgsMachine *machine)
 			    0, 0,
 			    (AGS_RECALL_FACTORY_INPUT |
 			     AGS_RECALL_FACTORY_ADD |
-			     AGS_RECALL_FACTORY_RECALL),
+			     AGS_RECALL_FACTORY_PLAY),
 			    0);
 
-  list = ags_recall_find_type(audio->recall, AGS_TYPE_PLAY_NOTATION_AUDIO_RUN);
+  g_object_get(audio,
+	       "play", &start_play,
+	       NULL);
+
+  play = ags_recall_find_type(start_play,
+			      AGS_TYPE_PLAY_NOTATION_AUDIO_RUN);
+
+  if(play != NULL){
+    recall_notation_audio_run = AGS_PLAY_NOTATION_AUDIO_RUN(play->data);
 
-  if(list != NULL){
-    recall_notation_audio_run = AGS_PLAY_NOTATION_AUDIO_RUN(list->data);
-    
     /* set dependency */
     g_object_set(G_OBJECT(recall_notation_audio_run),
 		 "delay-audio-run", play_delay_audio_run,
@@ -598,6 +627,8 @@ ags_syncsynth_map_recall(AgsMachine *machine)
 		 NULL);
   }
 
+  g_list_free(start_play);
+
   /* depending on destination */
   ags_syncsynth_input_map_recall(syncsynth, 0);
 
@@ -748,76 +779,22 @@ ags_syncsynth_resize_audio_channels(AgsMachine *machine,
   AgsSyncsynth *syncsynth;
 
   AgsAudio *audio;
-  
-  AgsMutexManager *mutex_manager;
-
-  AgsConfig *config;
-
-  gchar *str;
 
   guint output_pads, input_pads;
-  gboolean rt_safe;
-  gboolean performance_mode;
-
-  pthread_mutex_t *application_mutex;
-  pthread_mutex_t *audio_mutex;
-
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-  
-  config = ags_config_get_instance();
 
   syncsynth = (AgsSyncsynth *) machine;
   
   audio = machine->audio;
 
-  /* get audio mutex */
-  pthread_mutex_lock(application_mutex);
-
-  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
-					 (GObject *) audio);
-  
-  pthread_mutex_unlock(application_mutex);  
-
   /* get some fields */
-  pthread_mutex_lock(audio_mutex);
-  
-  output_pads = audio->output_pads;
-  input_pads = audio->input_pads;
-
-  pthread_mutex_unlock(audio_mutex);
+  g_object_get(audio,
+	       "output-pads", &output_pads,
+	       "input-pads", &input_pads,
+	       NULL);
 
   if(audio_channels > audio_channels_old){
-    /* map dependending on output */
-    rt_safe = TRUE;
-    performance_mode = TRUE;
-
-    str = ags_config_get_value(config,
-			       AGS_CONFIG_GENERIC,
-			       "rt-safe");
-
-    if(str != NULL &&
-       !g_ascii_strncasecmp(str,
-			    "FALSE",
-			    6)){
-      rt_safe = FALSE;
-    }
-
-    str = ags_config_get_value(config,
-			       AGS_CONFIG_GENERIC,
-			       "engine-mode");
-
-    if(str != NULL &&
-       !g_ascii_strncasecmp(str,
-			    "performance",
-			    12)){
-      performance_mode = TRUE;
-    }else{
-      performance_mode = FALSE;
-    }
-
-    if(rt_safe ||
-       performance_mode){
+    if(ags_recall_global_get_rt_safe() ||
+       ags_recall_global_get_performance_mode()){
       /* ags-copy */
       ags_recall_factory_create(audio,
 				NULL, NULL,
@@ -828,9 +805,6 @@ ags_syncsynth_resize_audio_channels(AgsMachine *machine,
 				 AGS_RECALL_FACTORY_RECALL |
 				 AGS_RECALL_FACTORY_ADD),
 				0);
-
-      /* set performance mode */
-      performance_mode = TRUE;
     }else{    
       /* ags-buffer */
       ags_recall_factory_create(audio,
@@ -868,7 +842,7 @@ ags_syncsynth_resize_audio_channels(AgsMachine *machine,
 			      0);
 
     /* ags-stream */
-    if(!rt_safe){
+    if(!ags_recall_global_get_rt_safe()){
       ags_recall_factory_create(audio,
 				NULL, NULL,
 				"ags-stream",
@@ -894,7 +868,8 @@ ags_syncsynth_resize_audio_channels(AgsMachine *machine,
     
     /* AgsOutput */
     /* ags-stream */
-    if(!performance_mode){
+    if(!(ags_recall_global_get_rt_safe() ||
+	 ags_recall_global_get_performance_mode())){
       ags_recall_factory_create(audio,
 				NULL, NULL,
 				"ags-stream",
@@ -920,39 +895,18 @@ ags_syncsynth_resize_pads(AgsMachine *machine, GType type,
   AgsChannel *channel, *source;
   AgsAudioSignal *audio_signal;
 
-  AgsMutexManager *mutex_manager;
-
-  AgsApplicationContext *application_context;
-  
   guint i, j;
   gboolean grow;
 
-  pthread_mutex_t *application_mutex;
-  pthread_mutex_t *audio_mutex;
-  pthread_mutex_t *source_mutex;
-
   if(pads == pads_old){
     return;
   }
 
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-
   window = (AgsWindow *) gtk_widget_get_toplevel((GtkWidget *) machine);
 
   syncsynth = (AgsSyncsynth *) machine;
 
   audio = machine->audio;
-
-  application_context = window->application_context;
-  
-  /* lookup audio mutex */
-  pthread_mutex_lock(application_mutex);
-    
-  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
-					 (GObject *) audio);
-  
-  pthread_mutex_unlock(application_mutex);
   
   if(pads_old < pads){
     grow = TRUE;
@@ -962,32 +916,6 @@ ags_syncsynth_resize_pads(AgsMachine *machine, GType type,
   
   if(type == AGS_TYPE_INPUT){
     if(grow){
-      pthread_mutex_lock(audio_mutex);
-
-      source = audio->input;
-
-      pthread_mutex_unlock(audio_mutex);
-      
-      /* create pattern */
-      source = ags_channel_nth(source, pads_old);
-      
-      while(source != NULL){
-	/* lookup source mutex */
-	pthread_mutex_lock(application_mutex);
-
-	source_mutex = ags_mutex_manager_lookup(mutex_manager,
-						(GObject *) source);
-  
-	pthread_mutex_unlock(application_mutex);
-
-	/* iterate pattern */
-	pthread_mutex_lock(source_mutex);
-	
-	source = source->next;
-
-	pthread_mutex_unlock(source_mutex);
-      }
-
       if((AGS_MACHINE_MAPPED_RECALL & (machine->flags)) != 0){
 	/* depending on destination */
 	ags_syncsynth_input_map_recall(syncsynth,
@@ -1001,11 +929,9 @@ ags_syncsynth_resize_pads(AgsMachine *machine, GType type,
     if(grow){
       AgsChannel *current, *output;
 
-      pthread_mutex_lock(audio_mutex);
-
-      source = audio->output;
-
-      pthread_mutex_unlock(audio_mutex);
+      g_object_get(audio,
+		   "output", &source,
+		   NULL);
 
       source = ags_channel_nth(source,
 			       pads_old);
@@ -1014,31 +940,19 @@ ags_syncsynth_resize_pads(AgsMachine *machine, GType type,
 	AgsRecycling *recycling;
 	AgsAudioSignal *audio_signal;
 
-	GObject *soundcard;
-
-	pthread_mutex_lock(audio_mutex);
+	GObject *output_soundcard;
 	
-	soundcard = audio->soundcard;
-
-	pthread_mutex_unlock(audio_mutex);
-
-	/* lookup source mutex */
-	pthread_mutex_lock(application_mutex);
-
-	source_mutex = ags_mutex_manager_lookup(mutex_manager,
-						(GObject *) source);
-  
-	pthread_mutex_unlock(application_mutex);
+	g_object_get(audio,
+		     "output-soundcard", &output_soundcard,
+		     NULL);
 
 	/* get recycling */
-	pthread_mutex_lock(source_mutex);
-
-	recycling = source->first_recycling;
-
-	pthread_mutex_unlock(source_mutex);
+	g_object_get(source,
+		     "first-recycling", &recycling,
+		     NULL);
 
 	/* instantiate template audio signal */
-	audio_signal = ags_audio_signal_new(soundcard,
+	audio_signal = ags_audio_signal_new(output_soundcard,
 					    (GObject *) recycling,
 					    NULL);
 	audio_signal->flags |= AGS_AUDIO_SIGNAL_TEMPLATE;
@@ -1063,80 +977,26 @@ ags_syncsynth_input_map_recall(AgsSyncsynth *syncsynth,
 {
   AgsAudio *audio;
 
-  AgsMutexManager *mutex_manager;  
-
-  AgsConfig *config;
-
   GList *list;
 
-  gchar *str;
-
   guint input_pads;
   guint audio_channels;
-  gboolean rt_safe;
-  gboolean performance_mode;
-
-  pthread_mutex_t *application_mutex;
-  pthread_mutex_t *audio_mutex;
 
   if(syncsynth->mapped_input_pad > input_pad_start){
     return;
   }
 
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-
-  config = ags_config_get_instance();
-
   audio = AGS_MACHINE(syncsynth)->audio;
 
-  /* get audio mutex */
-  pthread_mutex_lock(application_mutex);
-
-  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
-					 (GObject *) audio);
-  
-  pthread_mutex_unlock(application_mutex);  
-
   /* get some fields */
-  pthread_mutex_lock(audio_mutex);
-
-  input_pads = audio->input_pads;
-  audio_channels = audio->audio_channels;
-  
-  pthread_mutex_unlock(audio_mutex);
-
-  /* map dependending on output */
-  rt_safe = TRUE;
-  performance_mode = TRUE;
-
-  str = ags_config_get_value(config,
-			     AGS_CONFIG_GENERIC,
-			     "rt-safe");
-
-  if(str != NULL &&
-     !g_ascii_strncasecmp(str,
-			  "FALSE",
-			  6)){
-    rt_safe = FALSE;
-  }
-
-  str = ags_config_get_value(config,
-			     AGS_CONFIG_GENERIC,
-			     "engine-mode");
-
-  if(str != NULL &&
-     !g_ascii_strncasecmp(str,
-			  "performance",
-			  12)){
-    performance_mode = TRUE;
-  }else{
-    performance_mode = FALSE;
-  }
+  g_object_get(audio,
+	       "input-pads", &input_pads,
+	       "audio-channels", &audio_channels,
+	       NULL);
 
   /* remap for input */
-  if(rt_safe ||
-     performance_mode){
+  if(ags_recall_global_get_rt_safe() ||
+     ags_recall_global_get_performance_mode()){
     /* ags-copy */
     ags_recall_factory_create(audio,
 			      NULL, NULL,
@@ -1147,9 +1007,6 @@ ags_syncsynth_input_map_recall(AgsSyncsynth *syncsynth,
 			       AGS_RECALL_FACTORY_RECALL |
 			       AGS_RECALL_FACTORY_ADD),
 			      0);
-    
-    /* set performance mode */
-    performance_mode = TRUE;
   }else{    
     /* ags-buffer */
     ags_recall_factory_create(audio,
@@ -1199,7 +1056,8 @@ ags_syncsynth_input_map_recall(AgsSyncsynth *syncsynth,
 			    0);
   
   /* ags-stream */
-  if(!rt_safe){
+  if(!(ags_recall_global_get_rt_safe() ||
+       ags_recall_global_get_performance_mode())){
     ags_recall_factory_create(audio,
 			      NULL, NULL,
 			      "ags-stream",
@@ -1232,83 +1090,26 @@ ags_syncsynth_output_map_recall(AgsSyncsynth *syncsynth,
 {
   AgsAudio *audio;
 
-  AgsMutexManager *mutex_manager;
-
-  AgsConfig *config;
-
-  gchar *str;
-
   guint input_pad_start;
   guint output_pads, input_pads;
   guint audio_channels;
-  gboolean rt_safe;
-  gboolean performance_mode;
-
-  pthread_mutex_t *application_mutex;
-  pthread_mutex_t *audio_mutex;
 
   if(syncsynth->mapped_output_pad > output_pad_start){
     return;
   }
 
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-
-  config = ags_config_get_instance();
-
   audio = AGS_MACHINE(syncsynth)->audio;
 
-  /* get audio mutex */
-  pthread_mutex_lock(application_mutex);
-
-  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
-					 (GObject *) audio);
-  
-  pthread_mutex_unlock(application_mutex);
-
   /* get some fields */
-  input_pad_start = 0;
+  g_object_get(audio,
+	       "output-pads", &output_pads,
+	       "input-pads", &input_pads,
+	       "audio-channels", &audio_channels,
+	       NULL);
   
-  pthread_mutex_lock(audio_mutex);
-
-  output_pads = audio->output_pads;
-  input_pads = audio->input_pads;
-  
-  audio_channels = audio->audio_channels;
-
-  pthread_mutex_unlock(audio_mutex);
-  
-  /* map dependending on output */
-  rt_safe = TRUE;
-  performance_mode = TRUE;
-
-  str = ags_config_get_value(config,
-			     AGS_CONFIG_GENERIC,
-			     "rt-safe");
-
-  if(str != NULL &&
-     !g_ascii_strncasecmp(str,
-			  "FALSE",
-			  6)){
-    rt_safe = FALSE;
-  }
-
-  str = ags_config_get_value(config,
-			     AGS_CONFIG_GENERIC,
-			     "engine-mode");
-
-  if(str != NULL &&
-     !g_ascii_strncasecmp(str,
-			  "performance",
-			  12)){
-    performance_mode = TRUE;
-  }else{
-    performance_mode = FALSE;
-  }
-
   /* remap for input */
-  if(rt_safe ||
-     performance_mode){
+  if(ags_recall_global_get_rt_safe() ||
+     ags_recall_global_get_performance_mode()){
     /* ags-copy */
     ags_recall_factory_create(audio,
 			      NULL, NULL,
@@ -1319,9 +1120,6 @@ ags_syncsynth_output_map_recall(AgsSyncsynth *syncsynth,
 			       AGS_RECALL_FACTORY_RECALL |
 			       AGS_RECALL_FACTORY_ADD),
 			      0);
-    
-    /* set performance mode */
-    performance_mode = TRUE;
   }else{    
     /* ags-buffer */
     ags_recall_factory_create(audio,
@@ -1335,7 +1133,8 @@ ags_syncsynth_output_map_recall(AgsSyncsynth *syncsynth,
 			      0);
   }
 
-  if(!performance_mode){
+  if(!(ags_recall_global_get_rt_safe() ||
+       ags_recall_global_get_performance_mode())){
     /* ags-stream */
     ags_recall_factory_create(audio,
 			      NULL, NULL,
@@ -1358,14 +1157,20 @@ ags_syncsynth_output_map_recall(AgsSyncsynth *syncsynth,
  * 
  * Add @oscillator to @synthsync.
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_syncsynth_add_oscillator(AgsSyncsynth *syncsynth,
 			     AgsOscillator *oscillator)
 {
+  AgsAudio *audio;
+  
   GtkHBox *hbox;
   GtkCheckButton *check_button;
+
+  audio = AGS_MACHINE(syncsynth)->audio;
+  ags_audio_add_synth_generator(audio,
+				ags_synth_generator_new());
   
   hbox = (GtkHBox *) gtk_hbox_new(FALSE,
 				  0);
@@ -1398,14 +1203,29 @@ ags_syncsynth_add_oscillator(AgsSyncsynth *syncsynth,
  * 
  * Remove nth oscillator.
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_syncsynth_remove_oscillator(AgsSyncsynth *syncsynth,
 				guint nth)
 {
+  AgsAudio *audio;
+  
   GList *list, *list_start;
+  GList *start_synth_generator;
+  
+  audio = AGS_MACHINE(syncsynth)->audio;
+  g_object_get(audio,
+	       "synth-generator", &start_synth_generator,
+	       NULL);
 
+  start_synth_generator = g_list_reverse(start_synth_generator);
+  ags_audio_remove_synth_generator(audio,
+				   g_list_nth_data(start_synth_generator,
+						   nth));
+
+  g_list_free(start_synth_generator);
+  
   list =
     list_start = gtk_container_get_children(syncsynth->oscillator);
 
@@ -1425,7 +1245,7 @@ ags_syncsynth_remove_oscillator(AgsSyncsynth *syncsynth,
  * 
  * Reset loop spin buttons.
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_syncsynth_reset_loop(AgsSyncsynth *syncsynth)
@@ -1469,7 +1289,7 @@ ags_syncsynth_reset_loop(AgsSyncsynth *syncsynth)
  * 
  * Update audio data.
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_syncsynth_update(AgsSyncsynth *syncsynth)
@@ -1483,17 +1303,18 @@ ags_syncsynth_update(AgsSyncsynth *syncsynth)
   AgsClearAudioSignal *clear_audio_signal;
   AgsApplySynth *apply_synth;
 
-  AgsMutexManager *mutex_manager;
-  AgsThread *main_loop;
   AgsGuiThread *gui_thread;
 
   AgsApplicationContext *application_context;
   
   GList *list, *list_start;
+  GList *start_synth_generator, *synth_generator;    
   GList *child_start;
   GList *task;
   
   guint input_lines;
+  guint buffer_size;
+  guint format;
   guint wave;
   guint attack, frame_count;
   gdouble frequency, phase, start_frequency;
@@ -1503,37 +1324,13 @@ ags_syncsynth_update(AgsSyncsynth *syncsynth)
   AgsComplex **sync_point;
   guint sync_point_count;
 
-  pthread_mutex_t *audio_mutex;
-  pthread_mutex_t *channel_mutex;
-  pthread_mutex_t *application_mutex;
-
   window = (AgsWindow *) gtk_widget_get_toplevel((GtkWidget *) syncsynth);
-  application_context = (AgsApplicationContext *) window->application_context;
 
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
+  application_context = (AgsApplicationContext *) window->application_context;
+  gui_thread = ags_ui_provider_get_gui_thread(AGS_UI_PROVIDER(application_context));
 
   audio = AGS_MACHINE(syncsynth)->audio;
 
-  /* get audio loop */
-  pthread_mutex_lock(application_mutex);
-
-  main_loop = (AgsThread *) application_context->main_loop;
-
-  pthread_mutex_unlock(application_mutex);
-
-  /* get task thread */
-  gui_thread = (AgsGuiThread *) ags_thread_find_type(main_loop,
-						       AGS_TYPE_GUI_THREAD);
-
-  /* lookup audio mutex */
-  pthread_mutex_lock(application_mutex);
-    
-  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
-					 (GObject *) audio);
-  
-  pthread_mutex_unlock(application_mutex);
-
   /*  */
   start_frequency = (gdouble) gtk_spin_button_get_value_as_float(syncsynth->lower);
 
@@ -1541,69 +1338,63 @@ ags_syncsynth_update(AgsSyncsynth *syncsynth)
   loop_end = (guint) gtk_spin_button_get_value_as_int(syncsynth->loop_end);
 
   /* clear input */
-  pthread_mutex_lock(audio_mutex);
-
-  channel = audio->input;
-
-  pthread_mutex_unlock(audio_mutex);
+  g_object_get(audio,
+	       "input", &channel,
+	       NULL);
 
   task = NULL;
 
   while(channel != NULL){
+    AgsRecycling *first_recycling;
     AgsAudioSignal *template;
+
+    GList *start_list;
     
-    /* lookup channel mutex */
-    pthread_mutex_lock(application_mutex);
-
-    channel_mutex = ags_mutex_manager_lookup(mutex_manager,
-					     (GObject *) channel);
-  
-    pthread_mutex_unlock(application_mutex);
+    g_object_get(channel,
+		 "first-recycling", &first_recycling,
+		 NULL);
 
+    g_object_get(first_recycling,
+		 "audio-signal", &start_list,
+		 NULL);
+    
     /* clear task */
-    pthread_mutex_lock(channel_mutex);
-
-    template = ags_audio_signal_get_template(channel->first_recycling->audio_signal);
-
-    pthread_mutex_unlock(channel_mutex);
+    template = ags_audio_signal_get_template(start_list);
 
     clear_audio_signal = ags_clear_audio_signal_new(template);
     task = g_list_prepend(task,
 			  clear_audio_signal);
+
+    g_list_free(start_list);
     
     /* iterate */
-    pthread_mutex_lock(channel_mutex);
-
-    channel = channel->next;
-
-    pthread_mutex_unlock(channel_mutex);
+    g_object_get(channel,
+		 "next", &channel,
+		 NULL);
   }
 
   /* write input */
   list =
     list_start = gtk_container_get_children(syncsynth->oscillator);
 
-  /* get soundcard */
-  pthread_mutex_lock(audio_mutex);
+  /* get some fields */
+  g_object_get(audio,
+	       "input", &channel,
+	       "input-lines", &input_lines,
+	       "synth-generator", &start_synth_generator,
+	       NULL);
 
-  channel = audio->input;
+  g_object_get(channel,
+	       "buffer-size", &buffer_size,
+	       "format", &format,
+	       NULL);
 
-  input_lines = audio->input_lines;
-  
-  pthread_mutex_unlock(audio_mutex);
+  synth_generator = start_synth_generator;
 
   while(list != NULL){
     guint i;
     gboolean do_sync;
     
-    /* lookup channel mutex */
-    pthread_mutex_lock(application_mutex);
-
-    channel_mutex = ags_mutex_manager_lookup(mutex_manager,
-					     (GObject *) channel);
-  
-    pthread_mutex_unlock(application_mutex);
-
     /* do it so */
     child_start = gtk_container_get_children(list->data);
 
@@ -1611,64 +1402,78 @@ ags_syncsynth_update(AgsSyncsynth *syncsynth)
 
     g_list_free(child_start);
         
-    wave = (guint) gtk_combo_box_get_active(oscillator->wave) + 1;
+    wave = (guint)  + 1;
     attack = (guint) gtk_spin_button_get_value_as_int(oscillator->attack);
     frame_count = (guint) gtk_spin_button_get_value_as_int(oscillator->frame_count);
     phase = (gdouble) gtk_spin_button_get_value_as_float(oscillator->phase);
     frequency = (gdouble) gtk_spin_button_get_value_as_float(oscillator->frequency);
     volume = (gdouble) gtk_spin_button_get_value_as_float(oscillator->volume);
 
-    apply_synth = ags_apply_synth_new(channel, input_lines,
-				      wave,
-				      attack % channel->buffer_size, frame_count,
-				      frequency, phase, start_frequency,
-				      volume,
-				      loop_start, loop_end);
-    g_object_set(apply_synth,
-		 "delay", (gdouble) attack / channel->buffer_size,
+    g_object_set(synth_generator->data,
+		 "format", format,
+		 "delay", (gdouble) attack / buffer_size,
+		 "attack", attack,
+		 "frame-count", frame_count,
+		 "oscillator", gtk_combo_box_get_active(oscillator->wave),
+		 "frequency", frequency,
+		 "phase", phase,
+		 "volume", volume,
 		 NULL);
 
     do_sync = gtk_toggle_button_get_active(oscillator->do_sync);
 
     if(do_sync){
       sync_point_count = oscillator->sync_point_count;
-
-      if(sync_point_count > 0){
-	sync_point = (AgsComplex **) malloc(sync_point_count * sizeof(AgsComplex *));
-      }else{
-	sync_point = NULL;
-      }
       
-      for(i = 0; i < sync_point_count; i++){
-	sync_point[i] = ags_complex_alloc();
+      /* free previous sync point */
+      if(AGS_SYNTH_GENERATOR(synth_generator->data)->sync_point != NULL){
+	for(i = 0; i < AGS_SYNTH_GENERATOR(synth_generator->data)->sync_point_count; i++){
+	  ags_complex_free(AGS_SYNTH_GENERATOR(synth_generator->data)->sync_point[i]);
+	}
 
-	sync_point[i][0][0] = gtk_spin_button_get_value(oscillator->sync_point[2 * i]);
-	sync_point[i][0][1] = gtk_spin_button_get_value(oscillator->sync_point[2 * i + 1]);
+	free(AGS_SYNTH_GENERATOR(synth_generator->data)->sync_point);
+      }
+
+      /* set new sync point */
+      if(sync_point_count > 0){
+	AGS_SYNTH_GENERATOR(synth_generator->data)->sync_point = (AgsComplex **) malloc(sync_point_count * sizeof(AgsComplex *));
+      }else{
+	AGS_SYNTH_GENERATOR(synth_generator->data)->sync_point = NULL;
+      }
+
+      AGS_SYNTH_GENERATOR(synth_generator->data)->sync_point_count = sync_point_count;
+
+      for(i = 0; i < sync_point_count; i++){
+	AGS_SYNTH_GENERATOR(synth_generator->data)->sync_point[i] = ags_complex_alloc();
+
+	AGS_SYNTH_GENERATOR(synth_generator->data)->sync_point[i][0][0] = gtk_spin_button_get_value(oscillator->sync_point[2 * i]);
+	AGS_SYNTH_GENERATOR(synth_generator->data)->sync_point[i][0][1] = gtk_spin_button_get_value(oscillator->sync_point[2 * i + 1]);
       }
     }else{
-      sync_point = NULL;
-      sync_point_count = NULL;
+      for(i = 0; i < AGS_SYNTH_GENERATOR(synth_generator->data)->sync_point_count; i++){
+	ags_complex_free(AGS_SYNTH_GENERATOR(synth_generator->data)->sync_point[i]);
+      }
+
+      free(AGS_SYNTH_GENERATOR(synth_generator->data)->sync_point);
+      
+      AGS_SYNTH_GENERATOR(synth_generator->data)->sync_point = NULL;
+      AGS_SYNTH_GENERATOR(synth_generator->data)->sync_point_count = 0;
     }
-    
-    g_object_set(apply_synth,
-		 "base-note", syncsynth->lower->adjustment->value,
-		 "do-sync", do_sync,
-		 "sync-point", sync_point,
-		 "sync-point-count", sync_point_count,
-		 NULL);
-    
+					
+    apply_synth = ags_apply_synth_new(synth_generator->data,
+				      channel,
+				      start_frequency, input_lines);
+        
     task = g_list_prepend(task,
 			  apply_synth);
 
     /* iterate */
-    pthread_mutex_lock(channel_mutex);
-
-    channel = channel->next;
-
-    pthread_mutex_unlock(channel_mutex);
+    synth_generator = synth_generator->next;
     
     list = list->next;
   }
+
+  g_list_free(start_synth_generator);
   
   g_list_free(list_start);
 
@@ -1680,11 +1485,11 @@ ags_syncsynth_update(AgsSyncsynth *syncsynth)
  * ags_syncsynth_new:
  * @soundcard: the assigned soundcard.
  *
- * Creates an #AgsSyncsynth
+ * Create a new instance of #AgsSyncsynth
  *
- * Returns: a new #AgsSyncsynth
+ * Returns: the new #AgsSyncsynth
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsSyncsynth*
 ags_syncsynth_new(GObject *soundcard)
@@ -1694,8 +1499,8 @@ ags_syncsynth_new(GObject *soundcard)
   syncsynth = (AgsSyncsynth *) g_object_new(AGS_TYPE_SYNCSYNTH,
 					    NULL);
 
-  g_object_set(G_OBJECT(AGS_MACHINE(syncsynth)->audio),
-	       "soundcard", soundcard,
+  g_object_set(AGS_MACHINE(syncsynth)->audio,
+	       "output-soundcard", soundcard,
 	       NULL);
 
   return(syncsynth);
