@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2015 Joël Krähemann
+ * Copyright (C) 2005-2018 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -30,7 +30,6 @@
 
 void ags_recall_audio_class_init(AgsRecallAudioClass *recall_audio);
 void ags_recall_audio_connectable_interface_init(AgsConnectableInterface *connectable);
-void ags_recall_audio_packable_interface_init(AgsPackableInterface *packable);
 void ags_recall_audio_init(AgsRecallAudio *recall_audio);
 void ags_recall_audio_set_property(GObject *gobject,
 				   guint prop_id,
@@ -40,20 +39,13 @@ void ags_recall_audio_get_property(GObject *gobject,
 				   guint prop_id,
 				   GValue *value,
 				   GParamSpec *param_spec);
-void ags_recall_audio_connect(AgsConnectable *connectable);
-void ags_recall_audio_disconnect(AgsConnectable *connectable);
-gboolean ags_recall_audio_pack(AgsPackable *packable, GObject *container);
-gboolean ags_recall_audio_unpack(AgsPackable *packable);
 void ags_recall_audio_dispose(GObject *gobject);
 void ags_recall_audio_finalize(GObject *gobject);
 
-void ags_recall_audio_load_automation(AgsRecall *recall,
-				      GList *automation_port);
-void ags_recall_audio_unload_automation(AgsRecall *recall);
 void ags_recall_audio_automate(AgsRecall *recall);
 AgsRecall* ags_recall_audio_duplicate(AgsRecall *recall,
 				      AgsRecallID *recall_id,
-				      guint *n_params, GParameter *parameter);
+				      guint *n_params, gchar **parameter_name, GValue *value);
 
 /**
  * SECTION:ags_recall_audio
@@ -72,7 +64,6 @@ enum{
 
 static gpointer ags_recall_audio_parent_class = NULL;
 static AgsConnectableInterface* ags_recall_audio_parent_connectable_interface;
-static AgsPackableInterface* ags_recall_audio_parent_packable_interface;
 
 GType
 ags_recall_audio_get_type()
@@ -80,7 +71,7 @@ ags_recall_audio_get_type()
   static volatile gsize g_define_type_id__volatile = 0;
 
   if(g_once_init_enter (&g_define_type_id__volatile)){
-    GType ags_type_recall_audio;
+    GType ags_type_recall_audio = 0;
 
     static const GTypeInfo ags_recall_audio_info = {
       sizeof (AgsRecallAudioClass),
@@ -100,12 +91,6 @@ ags_recall_audio_get_type()
       NULL, /* interface_data */
     };
 
-    static const GInterfaceInfo ags_packable_interface_info = {
-      (GInterfaceInitFunc) ags_recall_audio_packable_interface_init,
-      NULL, /* interface_finalize */
-      NULL, /* interface_data */
-    };
-
     ags_type_recall_audio = g_type_register_static(AGS_TYPE_RECALL,
 						   "AgsRecallAudio",
 						   &ags_recall_audio_info,
@@ -115,11 +100,7 @@ ags_recall_audio_get_type()
 				AGS_TYPE_CONNECTABLE,
 				&ags_connectable_interface_info);
 
-    g_type_add_interface_static(ags_type_recall_audio,
-				AGS_TYPE_PACKABLE,
-				&ags_packable_interface_info);
-
-    g_once_init_leave (&g_define_type_id__volatile, ags_type_recall_audio);
+    g_once_init_leave(&g_define_type_id__volatile, ags_type_recall_audio);
   }
 
   return g_define_type_id__volatile;
@@ -149,7 +130,7 @@ ags_recall_audio_class_init(AgsRecallAudioClass *recall_audio)
    *
    * The assigned audio.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_object("audio",
 				   i18n_pspec("assigned audio"),
@@ -163,9 +144,8 @@ ags_recall_audio_class_init(AgsRecallAudioClass *recall_audio)
   /* AgsRecallClass */
   recall = (AgsRecallClass *) recall_audio;
 
-  recall->load_automation = ags_recall_audio_load_automation;
-  recall->unload_automation = ags_recall_audio_unload_automation;
   recall->automate = ags_recall_audio_automate;
+
   recall->duplicate = ags_recall_audio_duplicate;
 }
 
@@ -173,18 +153,6 @@ void
 ags_recall_audio_connectable_interface_init(AgsConnectableInterface *connectable)
 {
   ags_recall_audio_parent_connectable_interface = g_type_interface_peek_parent(connectable);
-
-  connectable->connect = ags_recall_audio_connect;
-  connectable->disconnect = ags_recall_audio_disconnect;
-}
-
-void
-ags_recall_audio_packable_interface_init(AgsPackableInterface *packable)
-{
-  ags_recall_audio_parent_packable_interface = g_type_interface_peek_parent(packable);
-
-  packable->pack = ags_recall_audio_pack;
-  packable->unpack = ags_recall_audio_unpack;
 }
 
 void
@@ -192,13 +160,6 @@ ags_recall_audio_init(AgsRecallAudio *recall_audio)
 {
   recall_audio->flags = 0;
 
-  recall_audio->samplerate = 0;
-  recall_audio->buffer_size = 0;
-  recall_audio->audio_buffer_util_format = 0;
-  
-  recall_audio->n_channels = 0;
-  recall_audio->mapping = NULL;
-  
   recall_audio->audio = NULL;
 }
 
@@ -210,18 +171,29 @@ ags_recall_audio_set_property(GObject *gobject,
 {
   AgsRecallAudio *recall_audio;
 
+  pthread_mutex_t *recall_mutex;
+
   recall_audio = AGS_RECALL_AUDIO(gobject);
 
+  /* get recall mutex */
+  pthread_mutex_lock(ags_recall_get_class_mutex());
+  
+  recall_mutex = AGS_RECALL(gobject)->obj_mutex;
+  
+  pthread_mutex_unlock(ags_recall_get_class_mutex());
+
   switch(prop_id){
   case PROP_AUDIO:
     {
       AgsAudio *audio;
-
-      guint i;
       
       audio = (AgsAudio *) g_value_get_object(value);
 
+      pthread_mutex_lock(recall_mutex);
+
       if(recall_audio->audio == audio){
+	pthread_mutex_unlock(recall_mutex);
+	
 	return;
       }
 
@@ -231,19 +203,11 @@ ags_recall_audio_set_property(GObject *gobject,
       
       if(audio != NULL){
 	g_object_ref(audio);
-
-	/* get audio channels */
-	recall_audio->n_channels = audio->audio_channels;
-	
-	/* allocate mapping */
-	recall_audio->mapping = (guint *) malloc(recall_audio->n_channels * sizeof(guint));
-
-	for(i = 0; i < recall_audio->n_channels; i++){
-	  recall_audio->mapping[i] = i;
-	}
       }
       
       recall_audio->audio = audio;
+
+      pthread_mutex_unlock(recall_mutex);
     }
     break;
   default:
@@ -260,11 +224,26 @@ ags_recall_audio_get_property(GObject *gobject,
 {
   AgsRecallAudio *recall_audio;
 
+  pthread_mutex_t *recall_mutex;
+
   recall_audio = AGS_RECALL_AUDIO(gobject);
 
+  /* get recall mutex */
+  pthread_mutex_lock(ags_recall_get_class_mutex());
+  
+  recall_mutex = AGS_RECALL(gobject)->obj_mutex;
+  
+  pthread_mutex_unlock(ags_recall_get_class_mutex());
+
   switch(prop_id){
   case PROP_AUDIO:
-    g_value_set_object(value, recall_audio->audio);
+    {
+      pthread_mutex_lock(recall_mutex);
+      
+      g_value_set_object(value, recall_audio->audio);
+
+      pthread_mutex_unlock(recall_mutex);
+    }
     break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID(gobject, prop_id, param_spec);
@@ -272,111 +251,12 @@ ags_recall_audio_get_property(GObject *gobject,
   }
 }
 
-void
-ags_recall_audio_connect(AgsConnectable *connectable)
-{
-  ags_recall_audio_parent_connectable_interface->connect(connectable);
-
-  /* empty */
-}
-
-void
-ags_recall_audio_disconnect(AgsConnectable *connectable)
-{
-  ags_recall_audio_parent_connectable_interface->disconnect(connectable);
-
-  /* empty */
-}
-
-gboolean
-ags_recall_audio_pack(AgsPackable *packable, GObject *container)
-{
-  AgsRecallContainer *recall_container;
-  GList *list;
-
-  if(ags_recall_audio_parent_packable_interface->pack(packable, container))
-    return(TRUE);
-
-  recall_container = AGS_RECALL_CONTAINER(container);
-
-  g_object_set(G_OBJECT(recall_container),
-	       "recall_audio", AGS_RECALL(packable),
-	       NULL);
-
-  /* set in AgsRecallAudioRun */
-  list = recall_container->recall_audio_run;
-
-  while(list != NULL){
-    g_object_set(G_OBJECT(list->data),
-		 "recall_audio", AGS_RECALL_AUDIO(packable),
-		 NULL);
-
-    list = list->next;
-  }
-
-  return(FALSE);
-}
-
-gboolean
-ags_recall_audio_unpack(AgsPackable *packable)
-{
-  AgsRecall *recall;
-  AgsRecallContainer *recall_container;
-  GList *list;
-
-  recall = AGS_RECALL(packable);
-
-  if(recall == NULL){
-    return(TRUE);
-  }
-
-  recall_container = AGS_RECALL_CONTAINER(recall->container);
-
-  if(recall_container == NULL){
-    return(TRUE);
-  }
-
-  /* ref */
-  g_object_ref(recall);
-  g_object_ref(recall_container);
-
-  /* unset in AgsRecallAudioRun */
-  list = recall_container->recall_audio_run;
-
-  while(list != NULL){
-    g_object_set(G_OBJECT(list->data),
-		 "recall_audio", NULL,
-		 NULL);
-
-    list = list->next;
-  }
-
-  /* call parent */
-  if(ags_recall_audio_parent_packable_interface->unpack(packable)){
-    g_object_unref(recall);
-    g_object_unref(recall_container);
-
-    return(TRUE);
-  }
-
-  recall_container->recall_audio = NULL;
-
-  /* unref */
-  g_object_unref(recall);
-  g_object_unref(recall_container);
-
-  return(FALSE);
-}
-
 void
 ags_recall_audio_dispose(GObject *gobject)
 {
   AgsRecallAudio *recall_audio;
 
   recall_audio = AGS_RECALL_AUDIO(gobject);
-
-  /* unpack */
-  ags_packable_unpack(AGS_PACKABLE(recall_audio));
   
   /* audio */
   if(recall_audio->audio != NULL){
@@ -400,112 +280,20 @@ ags_recall_audio_finalize(GObject *gobject)
   if(recall_audio->audio != NULL){
     g_object_unref(G_OBJECT(recall_audio->audio));
   }
-
-  if(recall_audio->mapping != NULL){
-    free(recall_audio->mapping);
-  }
   
   /* call parent */
   G_OBJECT_CLASS(ags_recall_audio_parent_class)->finalize(gobject);
 }
 
-void
-ags_recall_audio_load_automation(AgsRecall *recall,
-				 GList *automation_port)
-{
-  AgsAudio *audio;
-
-  GList *automation;
-  
-  audio = AGS_RECALL_AUDIO(recall)->audio;
-
-  while(automation_port != NULL){
-    if((AGS_PORT_IS_OUTPUT & (AGS_PORT(automation_port->data)->flags)) != 0){
-      automation_port = automation_port->next;
-
-      continue;
-    }
-
-    if(ags_automation_find_port(audio->automation,
-				automation_port->data) == NULL){
-      AgsAutomation *current;
-
-      current = ags_automation_new((GObject *) audio,
-				   0,
-				   G_TYPE_NONE,
-				   AGS_PORT(automation_port->data)->specifier);
-      g_object_set(current,
-		   "port", automation_port->data,
-		   NULL);
-      ags_audio_add_automation(audio,
-			       (GObject *) current);
-
-      //TODO:JK: property
-      AGS_PORT(automation_port->data)->automation = ags_automation_add(AGS_PORT(automation_port->data)->automation,
-								       current);
-    }else{
-      //TODO:JK: property
-      if(g_list_find(AGS_PORT(automation_port->data)->automation, automation->data) == NULL){
-	AGS_PORT(automation_port->data)->automation = ags_automation_add(AGS_PORT(automation_port->data)->automation,
-									 automation->data);
-      }
-    }
-    
-    automation_port = automation_port->next;
-  }
-
-  if(recall->automation_port == NULL){
-    recall->automation_port = automation_port;
-  }else{
-    recall->automation_port = g_list_concat(recall->automation_port,
-					    automation_port);
-  }
-}
-
-void
-ags_recall_audio_unload_automation(AgsRecall *recall)
-{
-  AgsAudio *audio;
-
-  AgsAutomation *current;
-  
-  GList *automation;
-  GList *automation_port;
-  
-  audio = AGS_RECALL_AUDIO(recall)->audio;
-
-  automation_port = recall->automation_port;
-  
-  while(automation_port != NULL){
-    automation = audio->automation;
-    
-    while((automation = ags_automation_find_port(automation,
-						 (GObject *) automation_port->data)) != NULL){
-      current = automation->data;
-      ags_audio_remove_automation(audio,
-				  (GObject *) current);
-
-      automation = automation->next;
-    }
-    
-    automation_port = automation_port->next;
-  }
-
-  g_list_free(recall->automation_port);
-  recall->automation_port = NULL;
-}
-
 void
 ags_recall_audio_automate(AgsRecall *recall)
 {
-  GObject *soundcard;
   AgsAudio *audio;
-  AgsAutomation *current;
 
-  AgsMutexManager *mutex_manager;
+  GObject *soundcard;
 
-  GList *automation;
-  GList *port;
+  GList *automation_start, *automation;
+  GList *port_start, *port;
 
   gdouble delay;
   guint note_offset, delay_counter;
@@ -517,66 +305,29 @@ ags_recall_audio_automate(AgsRecall *recall)
   guint ret_x;
   gboolean return_prev_on_failure;
 
-  pthread_mutex_t *application_mutex;
-  pthread_mutex_t *soundcard_mutex;
-  pthread_mutex_t *audio_mutex;
-  pthread_mutex_t *recall_mutex;
-
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-
-  /* recall mutex */
-  pthread_mutex_lock(ags_recall_get_class_mutex());
-
-  recall_mutex = recall->obj_mutex;
+  g_object_get(recall,
+	       "audio", &audio,
+	       NULL);
   
-  pthread_mutex_unlock(ags_recall_get_class_mutex());
-
-  /* get audio */
-  pthread_mutex_lock(recall_mutex);
-
-  audio = AGS_RECALL_AUDIO(recall)->audio;
-
-  pthread_mutex_unlock(recall_mutex);
-
-  /* audio mutex */
-  pthread_mutex_lock(application_mutex);
-
-  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
-					 (GObject *) audio);
+  g_object_get(audio,
+	       "soundcard", &soundcard,
+	       NULL);
   
-  pthread_mutex_unlock(application_mutex);
-
-  /* get soundcard */
-  pthread_mutex_lock(audio_mutex);
-
-  soundcard = audio->soundcard;
-
-  pthread_mutex_unlock(audio_mutex);
-
-  /* soundcard mutex */
-  pthread_mutex_lock(application_mutex);
-
-  soundcard_mutex = ags_mutex_manager_lookup(mutex_manager,
-					     (GObject *) soundcard);
-  
-  pthread_mutex_unlock(application_mutex);
-
-  /* retrieve position and loop information */
-  pthread_mutex_lock(soundcard_mutex);
+  g_object_get(recall,
+	       "port", &port_start,
+	       NULL);
 
+  /* retrieve position */
   note_offset = ags_soundcard_get_note_offset(AGS_SOUNDCARD(soundcard));
   
   delay = ags_soundcard_get_delay(AGS_SOUNDCARD(soundcard));
   delay_counter = ags_soundcard_get_delay_counter(AGS_SOUNDCARD(soundcard));
 
+  /* retrieve loop information */
   ags_soundcard_get_loop(AGS_SOUNDCARD(soundcard),
 			 &loop_left, &loop_right,
 			 &do_loop);
 
-  pthread_mutex_unlock(soundcard_mutex);
-
-  /*  */
   return_prev_on_failure = TRUE;
 
   if(do_loop &&
@@ -586,69 +337,99 @@ ags_recall_audio_automate(AgsRecall *recall)
     }
   }
 
-  /*  */
+  /* apply automation */
+  port = port_start;
+  
   x = ((double) note_offset + (delay_counter / delay)) * ((1.0 / AGS_AUTOMATION_MINIMUM_ACCELERATION_LENGTH) * AGS_NOTATION_MINIMUM_NOTE_LENGTH);
   step = ((1.0 / AGS_AUTOMATION_MINIMUM_ACCELERATION_LENGTH) * AGS_NOTATION_MINIMUM_NOTE_LENGTH);
 
-  pthread_mutex_lock(audio_mutex);
-
-  port = recall->port;
-
   while(port != NULL){
-    automation = (AgsAutomation *) AGS_PORT(port->data)->automation;
+    g_object_get(AGS_PORT(port->data),
+		 "automation", &automation_start,
+		 NULL);
+
+    /* find offset */
+    automation = automation_start;
 
     while(automation != NULL){
-      current = automation->data;
+      AgsAutomation *current_automation;
 
-      if(current->timestamp->timer.ags_offset.offset + AGS_AUTOMATION_DEFAULT_OFFSET < x){
+      AgsTimestamp *timestamp;
+      
+      guint current_automation_flags;
+      
+      pthread_mutex_t *automation_mutex;
+
+      current_automation = automation->data;
+
+      /* get automation mutex */
+      pthread_mutex_lock(ags_automation_get_class_mutex());
+
+      automation_mutex = current_automation->obj_mutex;
+      
+      pthread_mutex_unlock(ags_automation_get_class_mutex());
+
+      /* get some fields */
+      pthread_mutex_lock(automation_mutex);
+
+      current_automation_flags = current_automation->flags;
+      
+      timestamp = current_automation->timestamp;
+
+      pthread_mutex_unlock(automation_mutex);
+      
+      if(ags_timestamp_get_ags_offset(timestamp) + AGS_AUTOMATION_DEFAULT_OFFSET < x){
 	automation = automation->next;
 	
 	continue;
       }
       
-      if((AGS_AUTOMATION_BYPASS & (current->flags)) == 0){
+      if((AGS_AUTOMATION_BYPASS & (current_automation_flags)) == 0){
 	GValue value = {0,};
 	
-	ret_x = ags_automation_get_value(current,
+	ret_x = ags_automation_get_value(current_automation,
 					 floor(x), ceil(x + step),
 					 return_prev_on_failure,
 					 &value);
 
 	if(ret_x != G_MAXUINT){
-	  //NOTE:JK: warning nested lock
 	  ags_port_safe_write(port->data,
 			      &value);
 	}
       }
 
-      if(current->timestamp->timer.ags_offset.offset > ceil(x + step)){
+      if(ags_timestamp_get_ags_offset(timestamp) > ceil(x + step)){
 	break;
       }
 
+      /* iterate */
       automation = automation->next;
     }
 
+    g_list_free(automation_start);
+    
+    /* iterate */
     port = port->next;
   }
 
-  pthread_mutex_unlock(audio_mutex);
+  g_list_free(port_start);
 }
 
 AgsRecall*
 ags_recall_audio_duplicate(AgsRecall *recall,
 			   AgsRecallID *recall_id,
-			   guint *n_params, GParameter *parameter)
+			   guint *n_params, gchar **parameter_name, GValue *value)
 {
-  AgsRecallAudio *copy;
+  AgsRecallAudio *copy_recall_audio;
 
   /* duplicate */
-  copy = AGS_RECALL_AUDIO(AGS_RECALL_CLASS(ags_recall_audio_parent_class)->duplicate(recall,
-										     recall_id,
-										     n_params, parameter));
-
-  g_message("ags warning - ags_recall_audio_duplicate: you shouldn't do this %s\n", G_OBJECT_TYPE_NAME(recall));
+  copy_recall_audio = AGS_RECALL_AUDIO(AGS_RECALL_CLASS(ags_recall_audio_parent_class)->duplicate(recall,
+												  recall_id,
+												  n_params, parameter_name, value));
   
-  return((AgsRecall *) copy);
+  g_warning("ags_recall_audio_duplicate - you shouldn't do this %s", G_OBJECT_TYPE_NAME(recall));
+  
+  return((AgsRecall *) copy_recall_audio);
 }
 
 /**
@@ -659,7 +440,7 @@ ags_recall_audio_duplicate(AgsRecall *recall,
  *
  * Returns: a new #AgsRecallAudio.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsRecallAudio*
 ags_recall_audio_new(AgsAudio *audio)
