@@ -117,13 +117,15 @@ enum{
 static gpointer ags_midi_parser_parent_class = NULL;
 static guint midi_parser_signals[LAST_SIGNAL];
 
+static pthread_mutex_t ags_midi_parser_class_mutex = PTHREAD_MUTEX_INITIALIZER;
+
 GType
 ags_midi_parser_get_type(void)
 {
   static volatile gsize g_define_type_id__volatile = 0;
 
   if(g_once_init_enter (&g_define_type_id__volatile)){
-    GType ags_type_midi_parser;
+    GType ags_type_midi_parser = 0;
 
     static const GTypeInfo ags_midi_parser_info = {
       sizeof (AgsMidiParserClass),
@@ -141,7 +143,7 @@ ags_midi_parser_get_type(void)
 						  "AgsMidiParser", &ags_midi_parser_info,
 						  0);
 
-    g_once_init_leave (&g_define_type_id__volatile, ags_type_midi_parser);
+    g_once_init_leave(&g_define_type_id__volatile, ags_type_midi_parser);
   }
 
   return g_define_type_id__volatile;
@@ -168,7 +170,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
    *
    * The file to parse data from.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_pointer("file",
 				    i18n_pspec("the file stream"),
@@ -218,7 +220,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
    *
    * Returns: The character read
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_parser_signals[MIDI_GETC] =
     g_signal_new("midi-getc",
@@ -226,7 +228,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiParserClass, midi_getc),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_INT__VOID,
+		 ags_cclosure_marshal_INT__VOID,
 		 G_TYPE_INT, 0);
 
   /**
@@ -235,7 +237,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
    *
    * The ::on-error signal is emited as error occurs.
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_parser_signals[ON_ERROR] =
     g_signal_new("on-error",
@@ -256,7 +258,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
    *
    * Returns: The XML node representing the event
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_parser_signals[PARSE_FULL] =
     g_signal_new("parse-full",
@@ -264,7 +266,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiParserClass, parse_full),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_POINTER__VOID,
+		 ags_cclosure_marshal_POINTER__VOID,
 		 G_TYPE_POINTER, 0);
 
   /**
@@ -277,7 +279,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
    *
    * Returns: The XML node representing the event
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_parser_signals[PARSE_BYTES] =
     g_signal_new("parse-bytes",
@@ -285,7 +287,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiParserClass, parse_bytes),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_POINTER__POINTER_UINT,
+		 ags_cclosure_marshal_POINTER__POINTER_UINT,
 		 G_TYPE_POINTER, 2,
 		 G_TYPE_POINTER,
 		 G_TYPE_UINT);
@@ -298,7 +300,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
    *
    * Returns: The XML node representing the header
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_parser_signals[PARSE_HEADER] =
     g_signal_new("parse-header",
@@ -306,7 +308,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiParserClass, parse_header),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_POINTER__VOID,
+		 ags_cclosure_marshal_POINTER__VOID,
 		 G_TYPE_POINTER, 0);
 
   /**
@@ -317,7 +319,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
    *
    * Returns: The XML node representing the track
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_parser_signals[PARSE_TRACK] =
     g_signal_new("parse-track",
@@ -325,7 +327,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiParserClass, parse_track),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_POINTER__VOID,
+		 ags_cclosure_marshal_POINTER__VOID,
 		 G_TYPE_POINTER, 0);
 
   /**
@@ -336,7 +338,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
    *
    * Returns: The XML node representing the event
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_parser_signals[KEY_ON] =
     g_signal_new("key-on",
@@ -344,7 +346,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiParserClass, key_on),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_POINTER__UINT,
+		 ags_cclosure_marshal_POINTER__UINT,
 		 G_TYPE_POINTER, 1,
 		 G_TYPE_UINT);
 
@@ -356,7 +358,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
    *
    * Returns: The XML node representing the event
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_parser_signals[KEY_OFF] =
     g_signal_new("key-off",
@@ -364,7 +366,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiParserClass, key_off),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_POINTER__UINT,
+		 ags_cclosure_marshal_POINTER__UINT,
 		 G_TYPE_POINTER, 1,
 		 G_TYPE_UINT);
 
@@ -376,7 +378,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
    *
    * Returns: The XML node representing the event
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_parser_signals[KEY_PRESSURE] =
     g_signal_new("key-pressure",
@@ -384,7 +386,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiParserClass, key_pressure),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_POINTER__UINT,
+		 ags_cclosure_marshal_POINTER__UINT,
 		 G_TYPE_POINTER, 1,
 		 G_TYPE_UINT);
 
@@ -396,7 +398,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
    *
    * Returns: The XML node representing the event
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_parser_signals[CHANGE_PARAMETER] =
     g_signal_new("change-parameter",
@@ -404,7 +406,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiParserClass, change_parameter),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_POINTER__UINT,
+		 ags_cclosure_marshal_POINTER__UINT,
 		 G_TYPE_POINTER, 1,
 		 G_TYPE_UINT);
 
@@ -416,7 +418,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
    *
    * Returns: The XML node representing the event
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_parser_signals[CHANGE_PITCH_BEND] =
     g_signal_new("change-pitch-bend",
@@ -424,7 +426,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiParserClass, change_pitch_bend),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_POINTER__UINT,
+		 ags_cclosure_marshal_POINTER__UINT,
 		 G_TYPE_POINTER, 1,
 		 G_TYPE_UINT);
 
@@ -436,7 +438,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
    *
    * Returns: The XML node representing the event
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_parser_signals[CHANGE_PROGRAM] =
     g_signal_new("change-program",
@@ -444,7 +446,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiParserClass, change_program),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_POINTER__UINT,
+		 ags_cclosure_marshal_POINTER__UINT,
 		 G_TYPE_POINTER, 1,
 		 G_TYPE_UINT);
 
@@ -456,7 +458,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
    *
    * Returns: The XML node representing the event
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_parser_signals[CHANGE_CHANNEL_PRESSURE] =
     g_signal_new("change-channel-pressure",
@@ -464,7 +466,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiParserClass, change_channel_pressure),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_POINTER__UINT,
+		 ags_cclosure_marshal_POINTER__UINT,
 		 G_TYPE_POINTER, 1,
 		 G_TYPE_UINT);
 
@@ -476,7 +478,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
    *
    * Returns: The XML node representing the event
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_parser_signals[NOT_DEFINED] =
     g_signal_new("not-defined",
@@ -484,7 +486,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiParserClass, not_defined),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_POINTER__UINT,
+		 ags_cclosure_marshal_POINTER__UINT,
 		 G_TYPE_POINTER, 1,
 		 G_TYPE_UINT);
 
@@ -496,7 +498,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
    *
    * Returns: The XML node representing the event
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_parser_signals[SYSEX] =
     g_signal_new("sysex",
@@ -504,7 +506,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiParserClass, sysex),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_POINTER__UINT,
+		 ags_cclosure_marshal_POINTER__UINT,
 		 G_TYPE_POINTER, 1,
 		 G_TYPE_UINT);
 
@@ -516,7 +518,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
    *
    * Returns: The XML node representing the event
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_parser_signals[SYSTEM_COMMON] =
     g_signal_new("system-common",
@@ -524,7 +526,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiParserClass, system_common),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_POINTER__UINT,
+		 ags_cclosure_marshal_POINTER__UINT,
 		 G_TYPE_POINTER, 1,
 		 G_TYPE_UINT);
 
@@ -536,7 +538,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
    *
    * Returns: The XML node representing the event
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_parser_signals[META_EVENT] =
     g_signal_new("meta-event",
@@ -544,7 +546,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiParserClass, meta_event),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_POINTER__UINT,
+		 ags_cclosure_marshal_POINTER__UINT,
 		 G_TYPE_POINTER, 1,
 		 G_TYPE_UINT);
 
@@ -556,7 +558,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
    *
    * Returns: The XML node representing the event
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_parser_signals[SEQUENCE_NUMBER] =
     g_signal_new("sequence-number",
@@ -564,7 +566,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiParserClass, sequence_number),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_POINTER__UINT,
+		 ags_cclosure_marshal_POINTER__UINT,
 		 G_TYPE_POINTER, 1,
 		 G_TYPE_UINT);
 
@@ -576,7 +578,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
    *
    * Returns: The XML node representing the event
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_parser_signals[END_OF_TRACK] =
     g_signal_new("end-of-track",
@@ -584,7 +586,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiParserClass, end_of_track),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_POINTER__UINT,
+		 ags_cclosure_marshal_POINTER__UINT,
 		 G_TYPE_POINTER, 1,
 		 G_TYPE_UINT);
 
@@ -596,7 +598,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
    *
    * Returns: The XML node representing the event
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_parser_signals[SMTPE] =
     g_signal_new("smtpe",
@@ -604,7 +606,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiParserClass, smtpe),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_POINTER__UINT,
+		 ags_cclosure_marshal_POINTER__UINT,
 		 G_TYPE_POINTER, 1,
 		 G_TYPE_UINT);
 
@@ -616,7 +618,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
    *
    * Returns: The XML node representing the event
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_parser_signals[TEMPO] =
     g_signal_new("tempo",
@@ -624,7 +626,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiParserClass, tempo),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_POINTER__UINT,
+		 ags_cclosure_marshal_POINTER__UINT,
 		 G_TYPE_POINTER, 1,
 		 G_TYPE_UINT);
 
@@ -636,7 +638,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
    *
    * Returns: The XML node representing the event
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_parser_signals[TIME_SIGNATURE] =
     g_signal_new("time-signature",
@@ -644,7 +646,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiParserClass, time_signature),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_POINTER__UINT,
+		 ags_cclosure_marshal_POINTER__UINT,
 		 G_TYPE_POINTER, 1,
 		 G_TYPE_UINT);
 
@@ -656,7 +658,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
    *
    * Returns: The XML node representing the event
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_parser_signals[KEY_SIGNATURE] =
     g_signal_new("key-signature",
@@ -664,7 +666,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiParserClass, key_signature),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_POINTER__UINT,
+		 ags_cclosure_marshal_POINTER__UINT,
 		 G_TYPE_POINTER, 1,
 		 G_TYPE_UINT);
 
@@ -676,7 +678,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
    *
    * Returns: The XML node representing the event
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_parser_signals[SEQUENCER_META_EVENT] =
     g_signal_new("sequencer-meta-event",
@@ -684,7 +686,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiParserClass, sequencer_meta_event),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_POINTER__UINT,
+		 ags_cclosure_marshal_POINTER__UINT,
 		 G_TYPE_POINTER, 1,
 		 G_TYPE_UINT);
 
@@ -696,7 +698,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
    *
    * Returns: The XML node representing the event
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_parser_signals[TEXT_EVENT] =
     g_signal_new("text-event",
@@ -704,7 +706,7 @@ ags_midi_parser_class_init(AgsMidiParserClass *midi_parser)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiParserClass, text_event),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_POINTER__UINT,
+		 ags_cclosure_marshal_POINTER__UINT,
 		 G_TYPE_POINTER, 1,
 		 G_TYPE_UINT);
 }
@@ -714,6 +716,21 @@ ags_midi_parser_init(AgsMidiParser *midi_parser)
 {
   midi_parser->flags = 0;
   
+  /* midi parser mutex */
+  midi_parser->obj_mutexattr = (pthread_mutexattr_t *) malloc(sizeof(pthread_mutexattr_t));
+  pthread_mutexattr_init(midi_parser->obj_mutexattr);
+  pthread_mutexattr_settype(midi_parser->obj_mutexattr,
+			    PTHREAD_MUTEX_RECURSIVE);
+
+#ifdef __linux__
+  pthread_mutexattr_setprotocol(midi_parser->obj_mutexattr,
+				PTHREAD_PRIO_INHERIT);
+#endif
+
+  midi_parser->obj_mutex =  (pthread_mutex_t *) malloc(sizeof(pthread_mutex_t));
+  pthread_mutex_init(midi_parser->obj_mutex,
+		     midi_parser->obj_mutexattr);
+
   midi_parser->file = NULL;
   midi_parser->nth_chunk = 0;
 
@@ -734,15 +751,29 @@ ags_midi_parser_set_property(GObject *gobject,
 {
   AgsMidiParser *midi_parser;
 
+  pthread_mutex_t *midi_parser_mutex;
+
   midi_parser = AGS_MIDI_PARSER(gobject);
+
+  /* get midi parser mutex */
+  pthread_mutex_lock(ags_midi_parser_get_class_mutex());
+  
+  midi_parser_mutex = midi_parser->obj_mutex;
+  
+  pthread_mutex_unlock(ags_midi_parser_get_class_mutex());
   
   switch(prop_id){
   case PROP_FILE:
     {
       struct stat sb;
+
+      pthread_mutex_lock(midi_parser_mutex);
       
       midi_parser->file = g_value_get_pointer(value);
 
+      pthread_mutex_unlock(midi_parser_mutex);
+
+      /* read file */
       fstat(fileno(midi_parser->file), &sb);
       
       midi_parser->file_length = sb.st_size;
@@ -766,13 +797,26 @@ ags_midi_parser_get_property(GObject *gobject,
 {
   AgsMidiParser *midi_parser;
 
+  pthread_mutex_t *midi_parser_mutex;
+
   midi_parser = AGS_MIDI_PARSER(gobject);
+
+  /* get midi parser mutex */
+  pthread_mutex_lock(ags_midi_parser_get_class_mutex());
+  
+  midi_parser_mutex = midi_parser->obj_mutex;
+  
+  pthread_mutex_unlock(ags_midi_parser_get_class_mutex());
   
   switch(prop_id){
   case PROP_FILE:
     {
+      pthread_mutex_lock(midi_parser_mutex);
+
       g_value_set_pointer(value,
 			  midi_parser->file);
+
+      pthread_mutex_unlock(midi_parser_mutex);
     }
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID(gobject, prop_id, param_spec);
@@ -783,9 +827,35 @@ ags_midi_parser_get_property(GObject *gobject,
 void
 ags_midi_parser_finalize(GObject *gobject)
 {
+  AgsMidiParser *midi_parser;
+    
+  midi_parser = (AgsMidiParser *) gobject;
+
+  pthread_mutex_destroy(midi_parser->obj_mutex);
+  free(midi_parser->obj_mutex);
+
+  pthread_mutexattr_destroy(midi_parser->obj_mutexattr);
+  free(midi_parser->obj_mutexattr);
+
+  /* call parent */
   G_OBJECT_CLASS(ags_midi_parser_parent_class)->finalize(gobject);
 }
 
+/**
+ * ags_midi_parser_get_class_mutex:
+ * 
+ * Use this function's returned mutex to access mutex fields.
+ *
+ * Returns: the class mutex
+ * 
+ * Since: 2.0.0
+ */
+pthread_mutex_t*
+ags_midi_parser_get_class_mutex()
+{
+  return(&ags_midi_parser_class_mutex);
+}
+
 /**
  * ags_midi_parser_read_gint16:
  * @midi_parser: the #AgsMidiParser
@@ -794,7 +864,7 @@ ags_midi_parser_finalize(GObject *gobject)
  * 
  * Returns: the read gint16
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 gint16
 ags_midi_parser_read_gint16(AgsMidiParser *midi_parser)
@@ -819,7 +889,7 @@ ags_midi_parser_read_gint16(AgsMidiParser *midi_parser)
  * 
  * Returns: the read gint32
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 gint32
 ags_midi_parser_read_gint24(AgsMidiParser *midi_parser)
@@ -847,7 +917,7 @@ ags_midi_parser_read_gint24(AgsMidiParser *midi_parser)
  * 
  * Returns: the read gint32
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 gint32
 ags_midi_parser_read_gint32(AgsMidiParser *midi_parser)
@@ -876,7 +946,7 @@ ags_midi_parser_read_gint32(AgsMidiParser *midi_parser)
  * 
  * Returns: the read varlength
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 long
 ags_midi_parser_read_varlength(AgsMidiParser *midi_parser)
@@ -911,7 +981,7 @@ ags_midi_parser_read_varlength(AgsMidiParser *midi_parser)
  * 
  * Returns: the text read as string
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 gchar*
 ags_midi_parser_read_text(AgsMidiParser *midi_parser,
@@ -955,7 +1025,7 @@ ags_midi_parser_read_text(AgsMidiParser *midi_parser,
  * 
  * Returns: the seconds read from ticks
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 gdouble
 ags_midi_parser_ticks_to_sec(AgsMidiParser *midi_parser,
@@ -1009,7 +1079,7 @@ ags_midi_parser_real_midi_getc(AgsMidiParser *midi_parser)
  * 
  * Returns: the byte read
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 int
 ags_midi_parser_midi_getc(AgsMidiParser *midi_parser)
@@ -1041,7 +1111,7 @@ ags_midi_parser_real_on_error(AgsMidiParser *midi_parser,
  * 
  * Error reporting.
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_parser_on_error(AgsMidiParser *midi_parser,
@@ -1111,7 +1181,7 @@ ags_midi_parser_real_parse_full(AgsMidiParser *midi_parser)
  * 
  * Returns: the xmlDoc representing your MIDI file
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 xmlDoc*  
 ags_midi_parser_parse_full(AgsMidiParser *midi_parser)
@@ -1228,7 +1298,7 @@ ags_midi_parser_real_parse_header(AgsMidiParser *midi_parser)
  * 
  * Returns: the xmlNode containing parsed data
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 xmlNode*  
 ags_midi_parser_parse_header(AgsMidiParser *midi_parser)
@@ -1391,7 +1461,7 @@ ags_midi_parser_real_parse_track(AgsMidiParser *midi_parser)
  * 
  * Returns: the xmlNode containing parsed data
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 xmlNode*  
 ags_midi_parser_parse_track(AgsMidiParser *midi_parser)
@@ -1418,7 +1488,7 @@ ags_midi_parser_parse_track(AgsMidiParser *midi_parser)
  * 
  * Returns: the xmlNode containing parsed data
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 xmlNode*
 ags_midi_parser_channel_message(AgsMidiParser *midi_parser, guint status)
@@ -1547,7 +1617,7 @@ ags_midi_parser_real_key_on(AgsMidiParser *midi_parser, guint status)
  * 
  * Returns: the xmlNode containing parsed data
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 xmlNode*  
 ags_midi_parser_key_on(AgsMidiParser *midi_parser, guint status)
@@ -1612,7 +1682,7 @@ ags_midi_parser_real_key_off(AgsMidiParser *midi_parser, guint status)
  * 
  * Returns: the xmlNode containing parsed data
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 xmlNode*  
 ags_midi_parser_key_off(AgsMidiParser *midi_parser, guint status)
@@ -1676,7 +1746,7 @@ ags_midi_parser_real_key_pressure(AgsMidiParser *midi_parser, guint status)
  * 
  * Returns: the xmlNode containing parsed data
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 xmlNode*
 ags_midi_parser_key_pressure(AgsMidiParser *midi_parser, guint status)
@@ -1823,7 +1893,7 @@ ags_midi_parser_real_change_parameter(AgsMidiParser *midi_parser, guint status)
  * 
  * Returns: the xmlNode containing parsed data
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 xmlNode*  
 ags_midi_parser_change_parameter(AgsMidiParser *midi_parser, guint status)
@@ -1887,7 +1957,7 @@ ags_midi_parser_real_change_pitch_bend(AgsMidiParser *midi_parser, guint status)
  * 
  * Returns: the xmlNode containing parsed data
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 xmlNode*  
 ags_midi_parser_change_pitch_bend(AgsMidiParser *midi_parser, guint status)
@@ -1946,7 +2016,7 @@ ags_midi_parser_real_change_program(AgsMidiParser *midi_parser, guint status)
  * 
  * Returns: the xmlNode containing parsed data
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 xmlNode*  
 ags_midi_parser_change_program(AgsMidiParser *midi_parser, guint status)
@@ -2006,7 +2076,7 @@ ags_midi_parser_real_change_channel_pressure(AgsMidiParser *midi_parser, guint s
  * 
  * Returns: the xmlNode containing parsed data
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 xmlNode*  
 ags_midi_parser_change_channel_pressure(AgsMidiParser *midi_parser, guint status)
@@ -2044,7 +2114,7 @@ ags_midi_parser_real_not_defined(AgsMidiParser *midi_parser, guint status)
  * 
  * Returns: the xmlNode containing parsed data
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 xmlNode*  
 ags_midi_parser_not_defined(AgsMidiParser *midi_parser, guint status)
@@ -2088,7 +2158,7 @@ ags_midi_parser_real_sysex(AgsMidiParser *midi_parser, guint status)
  * 
  * Returns: the xmlNode containing parsed data
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 xmlNode*
 ags_midi_parser_sysex(AgsMidiParser *midi_parser, guint status)
@@ -2195,7 +2265,7 @@ ags_midi_parser_real_system_common(AgsMidiParser *midi_parser, guint status)
  * 
  * Returns: the xmlNode containing parsed data
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 xmlNode*
 ags_midi_parser_system_common(AgsMidiParser *midi_parser, guint status)
@@ -2319,7 +2389,7 @@ ags_midi_parser_real_meta_event(AgsMidiParser *midi_parser, guint status)
  * 
  * Returns: the xmlNode containing parsed data
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 xmlNode*  
 ags_midi_parser_meta_event(AgsMidiParser *midi_parser, guint status)
@@ -2373,7 +2443,7 @@ ags_midi_parser_real_sequence_number(AgsMidiParser *midi_parser, guint meta_type
  * 
  * Returns: the xmlNode containing parsed data
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 xmlNode*  
 ags_midi_parser_sequence_number(AgsMidiParser *midi_parser, guint meta_type)
@@ -2424,7 +2494,7 @@ ags_midi_parser_real_end_of_track(AgsMidiParser *midi_parser, guint meta_type)
  * 
  * Returns: the xmlNode containing parsed data
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 xmlNode*  
 ags_midi_parser_end_of_track(AgsMidiParser *midi_parser, guint meta_type)
@@ -2494,7 +2564,7 @@ ags_midi_parser_real_smtpe(AgsMidiParser *midi_parser, guint meta_type)
  * 
  * Returns: the xmlNode containing parsed data
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 xmlNode*  
 ags_midi_parser_smtpe(AgsMidiParser *midi_parser, guint meta_type)
@@ -2552,7 +2622,7 @@ ags_midi_parser_real_tempo(AgsMidiParser *midi_parser, guint meta_type)
  * 
  * Returns: the xmlNode containing parsed data
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 xmlNode*  
 ags_midi_parser_tempo(AgsMidiParser *midi_parser, guint meta_type)
@@ -2618,7 +2688,7 @@ ags_midi_parser_real_time_signature(AgsMidiParser *midi_parser, guint meta_type)
  * 
  * Returns: the xmlNode containing parsed data
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 xmlNode*  
 ags_midi_parser_time_signature(AgsMidiParser *midi_parser, guint meta_type)
@@ -2676,7 +2746,7 @@ ags_midi_parser_real_key_signature(AgsMidiParser *midi_parser, guint meta_type)
  * 
  * Returns: the xmlNode containing parsed data
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 xmlNode*  
 ags_midi_parser_key_signature(AgsMidiParser *midi_parser, guint meta_type)
@@ -2833,7 +2903,7 @@ ags_midi_parser_real_text_event(AgsMidiParser *midi_parser, guint meta_type)
  * 
  * Returns: the xmlNode containing parsed data
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 xmlNode*  
 ags_midi_parser_text_event(AgsMidiParser *midi_parser, guint meta_type)
@@ -2852,6 +2922,16 @@ ags_midi_parser_text_event(AgsMidiParser *midi_parser, guint meta_type)
   return(node);
 }
 
+/**
+ * ags_midi_parser_new:
+ * @file: the FILE handle
+ * 
+ * Creates a new instance of #AgsMidiParser
+ *
+ * Returns: the new #AgsMidiParser
+ * 
+ * Since: 2.0.0
+ */
 AgsMidiParser*
 ags_midi_parser_new(FILE *file)
 {
