@@ -38,10 +38,10 @@
 
 #define AGS_MIDI_PARSER_MAX_TEXT_LENGTH (4096)
 
-#define AGS_MIDI_PARSER_MTHD "MThd\0"
-#define AGS_MIDI_PARSER_MTCK "MTrk\0"
+#define AGS_MIDI_PARSER_MTHD "MThd"
+#define AGS_MIDI_PARSER_MTCK "MTrk"
 
-#define AGS_MIDI_EVENT "event\0"
+#define AGS_MIDI_EVENT "event"
 
 typedef struct _AgsMidiParser AgsMidiParser;
 typedef struct _AgsMidiParserClass AgsMidiParserClass;
@@ -63,6 +63,9 @@ struct _AgsMidiParser
 
   guint flags;
 
+  pthread_mutex_t *obj_mutex;
+  pthread_mutexattr_t *obj_mutexattr;
+
   FILE *file;
   guint nth_chunk;
 
@@ -119,6 +122,8 @@ struct _AgsMidiParserClass
 
 GType ags_midi_parser_get_type(void);
 
+pthread_mutex_t* ags_midi_parser_get_class_mutex();
+
 gint16 ags_midi_parser_read_gint16(AgsMidiParser *midi_parser);
 gint32 ags_midi_parser_read_gint24(AgsMidiParser *midi_parser);
 gint32 ags_midi_parser_read_gint32(AgsMidiParser *midi_parser);
