@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2017 Joël Krähemann
+ * Copyright (C) 2005-2018 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -18,17 +18,16 @@
  */
 
 #include <ags/audio/recall/ags_copy_pattern_audio_run.h>
-#include <ags/audio/recall/ags_copy_pattern_audio.h>
 
 #include <ags/libags.h>
 
 #include <ags/audio/ags_recall_container.h>
 
+#include <ags/audio/recall/ags_copy_pattern_audio.h>
+
 #include <ags/i18n.h>
 
 void ags_copy_pattern_audio_run_class_init(AgsCopyPatternAudioRunClass *copy_pattern_audio_run);
-void ags_copy_pattern_audio_run_connectable_interface_init(AgsConnectableInterface *connectable);
-void ags_copy_pattern_audio_run_dynamic_connectable_interface_init(AgsDynamicConnectableInterface *dynamic_connectable);
 void ags_copy_pattern_audio_run_plugin_interface_init(AgsPluginInterface *plugin);
 void ags_copy_pattern_audio_run_init(AgsCopyPatternAudioRun *copy_pattern_audio_run);
 void ags_copy_pattern_audio_run_set_property(GObject *gobject,
@@ -41,21 +40,16 @@ void ags_copy_pattern_audio_run_get_property(GObject *gobject,
 					     GParamSpec *param_spec);
 void ags_copy_pattern_audio_run_dispose(GObject *gobject);
 void ags_copy_pattern_audio_run_finalize(GObject *gobject);
-void ags_copy_pattern_audio_run_connect(AgsConnectable *connectable);
-void ags_copy_pattern_audio_run_disconnect(AgsConnectable *connectable);
-void ags_copy_pattern_audio_run_connect_dynamic(AgsDynamicConnectable *dynamic_connectable);
-void ags_copy_pattern_audio_run_disconnect_dynamic(AgsDynamicConnectable *dynamic_connectable);
+
 void ags_copy_pattern_audio_run_read(AgsFile *file, xmlNode *node, AgsPlugin *plugin);
 xmlNode* ags_copy_pattern_audio_run_write(AgsFile *file, xmlNode *parent, AgsPlugin *plugin);
 
-void ags_copy_pattern_audio_run_resolve_dependencies(AgsRecall *recall);
-AgsRecall* ags_copy_pattern_audio_run_duplicate(AgsRecall *recall,
-						AgsRecallID *recall_id,
-						guint *n_params, GParameter *parameter);
-void ags_copy_pattern_audio_run_notify_dependency(AgsRecall *recall, guint notify_mode, gint count);
+void ags_copy_pattern_audio_run_resolve_dependency(AgsRecall *recall);
+void ags_copy_pattern_audio_run_notify_dependency(AgsRecall *recall,
+						  guint dependency, gboolean increase);
 
 void ags_copy_pattern_audio_run_write_resolve_dependency(AgsFileLookup *file_lookup,
-							GObject *recall);
+							 GObject *recall);
 void ags_copy_pattern_audio_run_read_resolve_dependency(AgsFileLookup *file_lookup,
 							GObject *recall);
 
@@ -76,8 +70,6 @@ enum{
 };
 
 static gpointer ags_copy_pattern_audio_run_parent_class = NULL;
-static AgsConnectableInterface* ags_copy_pattern_audio_run_parent_connectable_interface;
-static AgsDynamicConnectableInterface *ags_copy_pattern_audio_run_parent_dynamic_connectable_interface;
 static AgsPluginInterface *ags_copy_pattern_audio_run_parent_plugin_interface;
 
 GType
@@ -86,32 +78,20 @@ ags_copy_pattern_audio_run_get_type()
   static volatile gsize g_define_type_id__volatile = 0;
 
   if(g_once_init_enter (&g_define_type_id__volatile)){
-    GType ags_type_copy_pattern_audio_run;
+    GType ags_type_copy_pattern_audio_run = 0;
 
     static const GTypeInfo ags_copy_pattern_audio_run_info = {
-      sizeof (AgsCopyPatternAudioRunClass),
+      sizeof(AgsCopyPatternAudioRunClass),
       NULL, /* base_init */
       NULL, /* base_finalize */
       (GClassInitFunc) ags_copy_pattern_audio_run_class_init,
       NULL, /* class_finalize */
       NULL, /* class_data */
-      sizeof (AgsCopyPatternAudioRun),
+      sizeof(AgsCopyPatternAudioRun),
       0,    /* n_preallocs */
       (GInstanceInitFunc) ags_copy_pattern_audio_run_init,
     };
 
-    static const GInterfaceInfo ags_connectable_interface_info = {
-      (GInterfaceInitFunc) ags_copy_pattern_audio_run_connectable_interface_init,
-      NULL, /* interface_finalize */
-      NULL, /* interface_data */
-    };
-
-    static const GInterfaceInfo ags_dynamic_connectable_interface_info = {
-      (GInterfaceInitFunc) ags_copy_pattern_audio_run_dynamic_connectable_interface_init,
-      NULL, /* interface_finalize */
-      NULL, /* interface_data */
-    };
-
     static const GInterfaceInfo ags_plugin_interface_info = {
       (GInterfaceInitFunc) ags_copy_pattern_audio_run_plugin_interface_init,
       NULL, /* interface_finalize */
@@ -123,19 +103,11 @@ ags_copy_pattern_audio_run_get_type()
 							     &ags_copy_pattern_audio_run_info,
 							     0);
     
-    g_type_add_interface_static(ags_type_copy_pattern_audio_run,
-				AGS_TYPE_CONNECTABLE,
-				&ags_connectable_interface_info);
-
-    g_type_add_interface_static(ags_type_copy_pattern_audio_run,
-				AGS_TYPE_DYNAMIC_CONNECTABLE,
-				&ags_dynamic_connectable_interface_info);
-
     g_type_add_interface_static(ags_type_copy_pattern_audio_run,
 				AGS_TYPE_PLUGIN,
 				&ags_plugin_interface_info);
 
-    g_once_init_leave (&g_define_type_id__volatile, ags_type_copy_pattern_audio_run);
+    g_once_init_leave(&g_define_type_id__volatile, ags_type_copy_pattern_audio_run);
   }
 
   return g_define_type_id__volatile;
@@ -166,7 +138,7 @@ ags_copy_pattern_audio_run_class_init(AgsCopyPatternAudioRunClass *copy_pattern_
    *
    * The delay audio run dependency.
    * 
-   * Since: 1.0.0.7
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_object("delay-audio-run",
 				   i18n_pspec("assigned AgsDelayAudioRun"),
@@ -182,7 +154,7 @@ ags_copy_pattern_audio_run_class_init(AgsCopyPatternAudioRunClass *copy_pattern_
    *
    * The count beats audio run dependency.
    * 
-   * Since: 1.0.0.7
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_object("count-beats-audio-run",
 				   i18n_pspec("assigned AgsCountBeatsAudioRun"),
@@ -196,29 +168,10 @@ ags_copy_pattern_audio_run_class_init(AgsCopyPatternAudioRunClass *copy_pattern_
   /* AgsRecallClass */
   recall = (AgsRecallClass *) copy_pattern_audio_run;
 
-  recall->resolve_dependencies = ags_copy_pattern_audio_run_resolve_dependencies;
-  recall->duplicate = ags_copy_pattern_audio_run_duplicate;
+  recall->resolve_dependency = ags_copy_pattern_audio_run_resolve_dependency;
   recall->notify_dependency = ags_copy_pattern_audio_run_notify_dependency;
 }
 
-void
-ags_copy_pattern_audio_run_connectable_interface_init(AgsConnectableInterface *connectable)
-{
-  ags_copy_pattern_audio_run_parent_connectable_interface = g_type_interface_peek_parent(connectable);
-
-  connectable->connect = ags_copy_pattern_audio_run_connect;
-  connectable->disconnect = ags_copy_pattern_audio_run_disconnect;
-}
-
-void
-ags_copy_pattern_audio_run_dynamic_connectable_interface_init(AgsDynamicConnectableInterface *dynamic_connectable)
-{
-  ags_copy_pattern_audio_run_parent_dynamic_connectable_interface = g_type_interface_peek_parent(dynamic_connectable);
-
-  dynamic_connectable->connect_dynamic = ags_copy_pattern_audio_run_connect_dynamic;
-  dynamic_connectable->disconnect_dynamic = ags_copy_pattern_audio_run_disconnect_dynamic;
-}
-
 void
 ags_copy_pattern_audio_run_plugin_interface_init(AgsPluginInterface *plugin)
 {
@@ -231,6 +184,8 @@ ags_copy_pattern_audio_run_plugin_interface_init(AgsPluginInterface *plugin)
 void
 ags_copy_pattern_audio_run_init(AgsCopyPatternAudioRun *copy_pattern_audio_run)
 {
+  ags_recall_set_ability_flags(copy_pattern_audio_run, (AGS_SOUND_ABILITY_SEQUENCER));
+
   AGS_RECALL(copy_pattern_audio_run)->name = "ags-copy-pattern";
   AGS_RECALL(copy_pattern_audio_run)->version = AGS_RECALL_DEFAULT_VERSION;
   AGS_RECALL(copy_pattern_audio_run)->build_id = AGS_RECALL_DEFAULT_BUILD_ID;
@@ -252,84 +207,121 @@ ags_copy_pattern_audio_run_set_property(GObject *gobject,
 {
   AgsCopyPatternAudioRun *copy_pattern_audio_run;
 
+  pthread_mutex_t *recall_mutex;
+  
   copy_pattern_audio_run = AGS_COPY_PATTERN_AUDIO_RUN(gobject);
 
+  /* get recall mutex */
+  pthread_mutex_lock(ags_recall_get_class_mutex());
+  
+  recall_mutex = AGS_RECALL(gobject)->obj_mutex;
+
+  pthread_mutex_unlock(ags_recall_get_class_mutex());
+
   switch(prop_id){
   case PROP_DELAY_AUDIO_RUN:
     {
-      AgsDelayAudioRun *delay_audio_run;
+      AgsDelayAudioRun *delay_audio_run, *old_delay_audio_run;
+
       gboolean is_template;
 
       delay_audio_run = (AgsDelayAudioRun *) g_value_get_object(value);
+      old_delay_audio_run = NULL;
+      
+      pthread_mutex_lock(recall_mutex);
+
+      if(copy_pattern_audio_run->delay_audio_run == delay_audio_run){
+	pthread_mutex_unlock(recall_mutex);
 
-      if(copy_pattern_audio_run->delay_audio_run == delay_audio_run)
 	return;
-
-      if(delay_audio_run != NULL &&
-	 (AGS_RECALL_TEMPLATE & (AGS_RECALL(delay_audio_run)->flags)) != 0){
-	is_template = TRUE;
-      }else{
-	is_template = FALSE;
       }
 
       if(copy_pattern_audio_run->delay_audio_run != NULL){
-	if(is_template){
-	  ags_recall_remove_dependency(AGS_RECALL(copy_pattern_audio_run),
-				       (AgsRecall *) copy_pattern_audio_run->delay_audio_run);
-	}
-
+	old_delay_audio_run = copy_pattern_audio_run->delay_audio_run;
+	
 	g_object_unref(G_OBJECT(copy_pattern_audio_run->delay_audio_run));
       }
 
       if(delay_audio_run != NULL){
 	g_object_ref(G_OBJECT(delay_audio_run));
-
-	if(is_template){
-	  ags_recall_add_dependency(AGS_RECALL(copy_pattern_audio_run),
-				    ags_recall_dependency_new((GObject *) delay_audio_run));
-	}
       }
 
       copy_pattern_audio_run->delay_audio_run = delay_audio_run;
-    }
-    break;
-  case PROP_COUNT_BEATS_AUDIO_RUN:
-    {
-      AgsCountBeatsAudioRun *count_beats_audio_run;
-      gboolean is_template;
 
-      count_beats_audio_run = (AgsCountBeatsAudioRun *) g_value_get_object(value);
+      pthread_mutex_unlock(recall_mutex);
 
-      if(copy_pattern_audio_run->count_beats_audio_run == count_beats_audio_run){
-	return;
-      }
-
-      if(count_beats_audio_run != NULL &&
-	 (AGS_RECALL_TEMPLATE & (AGS_RECALL(count_beats_audio_run)->flags)) != 0){
+      /* dependency */
+      if(ags_recall_test_flags(delay_audio_run, AGS_RECALL_TEMPLATE)){
 	is_template = TRUE;
       }else{
 	is_template = FALSE;
       }
 
-      if(copy_pattern_audio_run->count_beats_audio_run != NULL){
-	if(is_template){
-	  ags_recall_remove_dependency(AGS_RECALL(copy_pattern_audio_run),
-				       (AgsRecall *) copy_pattern_audio_run->count_beats_audio_run);
-	}
+      if(is_template &&
+	 old_delay_audio_run != NULL){
+	ags_recall_remove_recall_dependency(AGS_RECALL(copy_pattern_audio_run),
+					    (AgsRecall *) old_delay_audio_run);
+      }
 
+      if(is_template &&
+	 delay_audio_run != NULL){
+	ags_recall_add_recall_dependency(AGS_RECALL(copy_pattern_audio_run),
+					 ags_recall_dependency_new((GObject *) delay_audio_run));
+      }
+    }
+    break;
+  case PROP_COUNT_BEATS_AUDIO_RUN:
+    {
+      AgsCountBeatsAudioRun *count_beats_audio_run, *old_count_beats_audio_run;
+
+      gboolean is_template;
+
+      count_beats_audio_run = (AgsCountBeatsAudioRun *) g_value_get_object(value);
+      old_count_beats_audio_run = NULL;
+      
+      pthread_mutex_lock(recall_mutex);
+
+      if(copy_pattern_audio_run->count_beats_audio_run == count_beats_audio_run){
+	pthread_mutex_unlock(recall_mutex);
+
+	return;
+      }
+
+      if(copy_pattern_audio_run->count_beats_audio_run != NULL){
+	old_count_beats_audio_run = copy_pattern_audio_run->count_beats_audio_run;
+	
 	g_object_unref(G_OBJECT(copy_pattern_audio_run->count_beats_audio_run));
       }
 
       if(count_beats_audio_run != NULL){
 	g_object_ref(G_OBJECT(count_beats_audio_run));
-
-	if(is_template){
-	  ags_recall_add_dependency(AGS_RECALL(copy_pattern_audio_run),
-				    ags_recall_dependency_new((GObject *) count_beats_audio_run));
-	}
       }
 
       copy_pattern_audio_run->count_beats_audio_run = count_beats_audio_run;
+
+      pthread_mutex_unlock(recall_mutex);
+
+      /* check template */
+      if(count_beats_audio_run != NULL &&
+	 ags_recall_test_flags(count_beats_audio_run, AGS_RECALL_TEMPLATE)){
+	is_template = TRUE;
+      }else{
+	is_template = FALSE;
+      }
+
+      /* dependency - remove */
+      if(is_template &&
+	 old_count_beats_audio_run != NULL){
+	ags_recall_remove_recall_dependency(AGS_RECALL(copy_pattern_audio_run),
+					    (AgsRecall *) old_count_beats_audio_run);
+      }
+      
+      /* dependency - add */
+      if(is_template &&
+	 count_beats_audio_run != NULL){
+	ags_recall_add_recall_dependency(AGS_RECALL(copy_pattern_audio_run),
+					 ags_recall_dependency_new((GObject *) count_beats_audio_run));
+      }
     }
     break;
   default:
@@ -346,17 +338,34 @@ ags_copy_pattern_audio_run_get_property(GObject *gobject,
 {
   AgsCopyPatternAudioRun *copy_pattern_audio_run;
   
+  pthread_mutex_t *recall_mutex;
+  
   copy_pattern_audio_run = AGS_COPY_PATTERN_AUDIO_RUN(gobject);
 
+  /* get recall mutex */
+  pthread_mutex_lock(ags_recall_get_class_mutex());
+  
+  recall_mutex = AGS_RECALL(gobject)->obj_mutex;
+
+  pthread_mutex_unlock(ags_recall_get_class_mutex());
+
   switch(prop_id){
   case PROP_DELAY_AUDIO_RUN:
     {
+      pthread_mutex_lock(recall_mutex);
+
       g_value_set_object(value, copy_pattern_audio_run->delay_audio_run);
+
+      pthread_mutex_unlock(recall_mutex);
     }
     break;
   case PROP_COUNT_BEATS_AUDIO_RUN:
     {
+      pthread_mutex_lock(recall_mutex);
+
       g_value_set_object(value, copy_pattern_audio_run->count_beats_audio_run);
+
+      pthread_mutex_unlock(recall_mutex);
     }
     break;
   default:
@@ -411,34 +420,6 @@ ags_copy_pattern_audio_run_finalize(GObject *gobject)
   G_OBJECT_CLASS(ags_copy_pattern_audio_run_parent_class)->finalize(gobject);
 }
 
-void
-ags_copy_pattern_audio_run_connect(AgsConnectable *connectable)
-{
-  /* call parent */
-  ags_copy_pattern_audio_run_parent_connectable_interface->connect(connectable);
-}
-
-void
-ags_copy_pattern_audio_run_disconnect(AgsConnectable *connectable)
-{
-  /* call parent */
-  ags_copy_pattern_audio_run_parent_connectable_interface->disconnect(connectable);
-}
-
-void
-ags_copy_pattern_audio_run_connect_dynamic(AgsDynamicConnectable *dynamic_connectable)
-{
-  /* call parent */
-  ags_copy_pattern_audio_run_parent_dynamic_connectable_interface->connect_dynamic(dynamic_connectable);
-}
-
-void
-ags_copy_pattern_audio_run_disconnect_dynamic(AgsDynamicConnectable *dynamic_connectable)
-{
-  /* call parent */
-  ags_copy_pattern_audio_run_parent_dynamic_connectable_interface->connect_dynamic(dynamic_connectable);
-}
-
 void
 ags_copy_pattern_audio_run_read(AgsFile *file, xmlNode *node, AgsPlugin *plugin)
 {
@@ -508,7 +489,7 @@ ags_copy_pattern_audio_run_write(AgsFile *file, xmlNode *parent, AgsPlugin *plug
   xmlAddChild(node,
 	      child);
 
-  list = AGS_RECALL(plugin)->dependencies;
+  list = AGS_RECALL(plugin)->recall_dependency;
 
   while(list != NULL){
     id = ags_id_generator_create_uuid();
@@ -538,101 +519,6 @@ ags_copy_pattern_audio_run_write(AgsFile *file, xmlNode *parent, AgsPlugin *plug
   return(node);
 }
 
-AgsRecall*
-ags_copy_pattern_audio_run_duplicate(AgsRecall *recall,
-				     AgsRecallID *recall_id,
-				     guint *n_params, GParameter *parameter)
-{
-  AgsCopyPatternAudioRun *copy;
-
-  copy = AGS_COPY_PATTERN_AUDIO_RUN(AGS_RECALL_CLASS(ags_copy_pattern_audio_run_parent_class)->duplicate(recall,
-													 recall_id,
-													 n_params, parameter));
-
-  return((AgsRecall *) copy);
-}
-
-void
-ags_copy_pattern_audio_run_notify_dependency(AgsRecall *recall, guint notify_mode, gint count)
-{
-  AgsCopyPatternAudioRun *copy_pattern_audio_run;
-
-  copy_pattern_audio_run = AGS_COPY_PATTERN_AUDIO_RUN(recall);
-
-  switch(notify_mode){
-  case AGS_RECALL_NOTIFY_RUN:
-    break;
-  case AGS_RECALL_NOTIFY_AUDIO:
-    break;
-  case AGS_RECALL_NOTIFY_AUDIO_RUN:
-    break;
-  case AGS_RECALL_NOTIFY_CHANNEL:
-    break;
-  case AGS_RECALL_NOTIFY_CHANNEL_RUN:
-    copy_pattern_audio_run->hide_ref += count;
-    g_message("copy_pattern_audio_run->hide_ref: %u\n", copy_pattern_audio_run->hide_ref);
-    break;
-  default:
-    g_message("ags_copy_pattern_audio_run.c - ags_copy_pattern_audio_run_notify: unknown notify");
-  }
-}
-
-void
-ags_copy_pattern_audio_run_resolve_dependencies(AgsRecall *recall)
-{
-  AgsRecall *template;
-  AgsRecallContainer *recall_container;
-  AgsRecallID *recall_id;
-  
-  AgsRecallDependency *recall_dependency;
-  AgsDelayAudioRun *delay_audio_run;
-  AgsCountBeatsAudioRun *count_beats_audio_run;
-
-  GList *list;
-  
-  guint i, i_stop;
-
-  recall_container = AGS_RECALL_CONTAINER(recall->container);
-  
-  list = ags_recall_find_template(recall_container->recall_audio_run);
-
-  if(list != NULL){
-    template = AGS_RECALL(list->data);
-  }else{
-    g_warning("AgsRecallClass::resolve - missing dependency");
-    return;
-  }
-
-  list = template->dependencies;
-  delay_audio_run = NULL;
-  count_beats_audio_run = NULL;
-  i_stop = 2;
-
-  for(i = 0; i < i_stop && list != NULL;){
-    recall_dependency = AGS_RECALL_DEPENDENCY(list->data);
-    recall_id = recall->recall_id;
-      
-    if(AGS_IS_DELAY_AUDIO_RUN(recall_dependency->dependency)){
-      delay_audio_run = (AgsDelayAudioRun *) ags_recall_dependency_resolve(recall_dependency,
-									   (AgsRecallID *) recall_id->recycling_context->parent->recall_id);
-
-      i++;
-    }else if(AGS_IS_COUNT_BEATS_AUDIO_RUN(recall_dependency->dependency)){
-      count_beats_audio_run = (AgsCountBeatsAudioRun *) ags_recall_dependency_resolve(recall_dependency,
-										      (AgsRecallID *) recall_id->recycling_context->parent->recall_id);
-
-      i++;
-    }
-
-    list = list->next;
-  }
-
-  g_object_set(G_OBJECT(recall),
-	       "delay-audio-run", delay_audio_run,
-	       "count-beats-audio-run", count_beats_audio_run,
-	       NULL);
-}
-
 void
 ags_copy_pattern_audio_run_write_resolve_dependency(AgsFileLookup *file_lookup,
 						    GObject *recall)
@@ -673,22 +559,169 @@ ags_copy_pattern_audio_run_read_resolve_dependency(AgsFileLookup *file_lookup,
   }
 }
 
+void
+ags_copy_pattern_audio_run_resolve_dependency(AgsRecall *recall)
+{
+  AgsRecall *template;
+  AgsRecallContainer *recall_container;
+  AgsRecallID *parent_recall_id;
+  AgsRecallID *recall_id;
+  AgsRecyclingContext *parent_recycling_context;
+  AgsRecyclingContext *recycling_context;
+  
+  AgsRecallDependency *recall_dependency;
+  AgsDelayAudioRun *delay_audio_run;
+  AgsCountBeatsAudioRun *count_beats_audio_run;
+
+  GObject *dependency;
+  
+  GList *list_start, *list;
+  
+  guint i, i_stop;
+
+  g_object_get(recall,
+	       "recall-container", &recall_container,
+	       NULL);
+
+  g_object_get(recall_container,
+	       "recall-audio-run", &list_start,
+	       NULL);
+  
+  list = ags_recall_find_template(list_start);
+
+  if(list == NULL){
+    g_warning("AgsRecallClass::resolve - missing dependency");
+
+    return;
+  }
+
+  template = AGS_RECALL(list->data);
+  g_list_free(list_start);
+  
+  g_object_get(template,
+	       "recall-dependency", &list_start,
+	       NULL);
+  
+  g_object_get(recall,
+	       "recall-id", &recall_id,
+	       NULL);
+
+  g_object_get(recall_id,
+	       "recycling-context", &recycling_context,
+	       NULL);
+
+  g_object_get(recycling_context,
+	       "parent", &parent_recycling_context,
+	       NULL);
+
+  g_object_get(parent_recycling_context,
+	       "recall-id", &parent_recall_id,
+	       NULL);
+  
+  /* prepare to resolve */
+  delay_audio_run = NULL;
+  count_beats_audio_run = NULL;
+
+  list = list_start;
+  
+  i_stop = 2;
+
+  for(i = 0; i < i_stop && list != NULL;){
+    recall_dependency = AGS_RECALL_DEPENDENCY(list->data);
+
+    g_object_get(recall_dependency,
+		 "dependency", &dependency,
+		 NULL);
+    
+    if(AGS_IS_DELAY_AUDIO_RUN(dependency)){
+      delay_audio_run = (AgsDelayAudioRun *) ags_recall_dependency_resolve(recall_dependency,
+									   parent_recall_id);
+
+      i++;
+    }else if(AGS_IS_COUNT_BEATS_AUDIO_RUN(dependency)){
+      count_beats_audio_run = (AgsCountBeatsAudioRun *) ags_recall_dependency_resolve(recall_dependency,
+										      parent_recall_id);
+
+      i++;
+    }
+
+    list = list->next;
+  }
+
+  g_list_free(list_start);
+  
+  g_object_set(G_OBJECT(recall),
+	       "delay-audio-run", delay_audio_run,
+	       "count-beats-audio-run", count_beats_audio_run,
+	       NULL);
+}
+
+void
+ags_copy_pattern_audio_run_notify_dependency(AgsRecall *recall,
+					     guint dependency, gboolean increase)
+{
+  AgsCopyPatternAudioRun *copy_pattern_audio_run;
+
+  pthread_mutex_t *recall_mutex;
+
+  copy_pattern_audio_run = AGS_COPY_PATTERN_AUDIO_RUN(recall);
+
+  /* get mutex */
+  pthread_mutex_lock(ags_recall_get_class_mutex());
+
+  recall_mutex = recall->obj_mutex;
+  
+  pthread_mutex_unlock(ags_recall_get_class_mutex());
+
+  /* notify */
+  pthread_mutex_lock(recall_mutex);
+
+  switch(dependency){
+  case AGS_RECALL_NOTIFY_RUN:
+    break;
+  case AGS_RECALL_NOTIFY_AUDIO:
+    break;
+  case AGS_RECALL_NOTIFY_AUDIO_RUN:
+    break;
+  case AGS_RECALL_NOTIFY_CHANNEL:
+    break;
+  case AGS_RECALL_NOTIFY_CHANNEL_RUN:
+    if(increase){
+      copy_pattern_audio_run->hide_ref += 1;
+    }else{
+      copy_pattern_audio_run->hide_ref -= 1;
+    }
+    break;
+  default:
+    g_message("ags_copy_pattern_audio_run.c - ags_copy_pattern_audio_run_notify: unknown notify");
+  }
+
+
+  pthread_mutex_unlock(recall_mutex);
+}
+
 /**
  * ags_copy_pattern_audio_run_new:
- * @count_beats_audio_run: an #AgsCountBeatsAudioRun as dependency
+ * @audio: the #AgsAudio
+ * @delay_audio_run: the #AgsDelayAudioRun dependency
+ * @count_beats_audio_run: the #AgsCountBeatsAudioRun dependency
  *
- * Creates an #AgsCopyPatternAudioRun
+ * Create a new instance of #AgsCopyPatternAudioRun
  *
- * Returns: a new #AgsCopyPatternAudioRun
+ * Returns: the new #AgsCopyPatternAudioRun
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsCopyPatternAudioRun*
-ags_copy_pattern_audio_run_new(AgsCountBeatsAudioRun *count_beats_audio_run)
+ags_copy_pattern_audio_run_new(AgsAudio *audio,
+			       AgsDelayAudioRun *delay_audio_run,
+			       AgsCountBeatsAudioRun *count_beats_audio_run)
 {
   AgsCopyPatternAudioRun *copy_pattern_audio_run;
 
   copy_pattern_audio_run = (AgsCopyPatternAudioRun *) g_object_new(AGS_TYPE_COPY_PATTERN_AUDIO_RUN,
+								   "audio", audio,
+								   "delay-audio-run", delay_audio_run,
 								   "count-beats-audio-run", count_beats_audio_run,
 								   NULL);
 
