@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2017 Joël Krähemann
+ * Copyright (C) 2005-2018 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -23,10 +23,7 @@
 
 #include <ags/plugin/ags_dssi_manager.h>
 #include <ags/plugin/ags_ladspa_conversion.h>
-
-#include <ags/file/ags_file.h>
-#include <ags/file/ags_file_stock.h>
-#include <ags/file/ags_file_id_ref.h>
+#include <ags/plugin/ags_plugin_port.h>
 
 #include <ags/audio/ags_port.h>
 
@@ -48,7 +45,6 @@
 #include <ags/i18n.h>
 
 void ags_play_dssi_audio_class_init(AgsPlayDssiAudioClass *play_dssi_audio);
-void ags_play_dssi_audio_connectable_interface_init(AgsConnectableInterface *connectable);
 void ags_play_dssi_audio_plugin_interface_init(AgsPluginInterface *plugin);
 void ags_play_dssi_audio_init(AgsPlayDssiAudio *play_dssi_audio);
 void ags_play_dssi_audio_set_property(GObject *gobject,
@@ -59,9 +55,9 @@ void ags_play_dssi_audio_get_property(GObject *gobject,
 				      guint prop_id,
 				      GValue *value,
 				      GParamSpec *param_spec);
+void ags_play_dssi_audio_dispose(GObject *gobject);
 void ags_play_dssi_audio_finalize(GObject *gobject);
-void ags_play_dssi_audio_connect(AgsConnectable *connectable);
-void ags_play_dssi_audio_disconnect(AgsConnectable *connectable);
+
 void ags_play_dssi_audio_set_ports(AgsPlugin *plugin, GList *port);
 
 /**
@@ -76,13 +72,12 @@ void ags_play_dssi_audio_set_ports(AgsPlugin *plugin, GList *port);
 
 enum{
   PROP_0,
-  PROP_FILENAME,
-  PROP_EFFECT,
-  PROP_INDEX,
+  PROP_PLUGIN,
+  PROP_INPUT_LINES,
+  PROP_OUTPUT_LINES,
 };
 
 static gpointer ags_play_dssi_audio_parent_class = NULL;
-static AgsConnectableInterface* ags_play_dssi_audio_parent_connectable_interface;
 static AgsPluginInterface *ags_play_dssi_parent_plugin_interface;
 
 static const gchar *ags_play_dssi_audio_plugin_name = "ags-play-dssi";
@@ -111,12 +106,6 @@ ags_play_dssi_audio_get_type()
       (GInstanceInitFunc) ags_play_dssi_audio_init,
     };
 
-    static const GInterfaceInfo ags_connectable_interface_info = {
-      (GInterfaceInitFunc) ags_play_dssi_audio_connectable_interface_init,
-      NULL, /* interface_finalize */
-      NULL, /* interface_data */
-    };
-
     static const GInterfaceInfo ags_plugin_interface_info = {
       (GInterfaceInitFunc) ags_play_dssi_audio_plugin_interface_init,
       NULL, /* interface_finalize */
@@ -128,10 +117,6 @@ ags_play_dssi_audio_get_type()
 						      &ags_play_dssi_audio_info,
 						      0);
 
-    g_type_add_interface_static(ags_type_play_dssi_audio,
-				AGS_TYPE_CONNECTABLE,
-				&ags_connectable_interface_info);
-
     g_type_add_interface_static(ags_type_play_dssi_audio,
 				AGS_TYPE_PLUGIN,
 				&ags_plugin_interface_info);
@@ -156,67 +141,61 @@ ags_play_dssi_audio_class_init(AgsPlayDssiAudioClass *play_dssi_audio)
   gobject->set_property = ags_play_dssi_audio_set_property;
   gobject->get_property = ags_play_dssi_audio_get_property;
 
+  gobject->dispose = ags_play_dssi_audio_dispose;
   gobject->finalize = ags_play_dssi_audio_finalize;
 
   /* properties */
   /**
-   * AgsPlayDssiAudio:filename:
+   * AgsPlayDssiAudio:plugin:
    *
-   * The plugins filename.
+   * The plugin's plugin object.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
-  param_spec =  g_param_spec_string("filename",
-				    i18n_pspec("the object file"),
-				    i18n_pspec("The filename as string of object file"),
-				    NULL,
+  param_spec =  g_param_spec_object("plugin",
+				    i18n_pspec("the plugin"),
+				    i18n_pspec("The plugin as plugin object"),
+				    AGS_TYPE_DSSI_PLUGIN,
 				    G_PARAM_READABLE | G_PARAM_WRITABLE);
   g_object_class_install_property(gobject,
-				  PROP_FILENAME,
+				  PROP_PLUGIN,
 				  param_spec);
 
   /**
-   * AgsPlayDssiAudio:effect:
+   * AgsPlayDssiAudio:input-lines:
    *
-   * The effect's name.
+   * The effect's input lines count.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
-  param_spec =  g_param_spec_string("effect",
-				    i18n_pspec("the effect"),
-				    i18n_pspec("The effect's string representation"),
-				    NULL,
-				    G_PARAM_READABLE | G_PARAM_WRITABLE);
-  g_object_class_install_property(gobject,
-				  PROP_EFFECT,
-				  param_spec);
-
-  /**
-   * AgsPlayDssiAudio:index:
-   *
-   * The effect's index.
-   * 
-   * Since: 1.0.0
-   */
-  param_spec =  g_param_spec_ulong("index",
-				   i18n_pspec("index of effect"),
-				   i18n_pspec("The numerical index of effect"),
+  param_spec =  g_param_spec_ulong("input-lines",
+				   i18n_pspec("input lines of effect"),
+				   i18n_pspec("The effect's count of input lines"),
 				   0,
 				   65535,
 				   0,
 				   G_PARAM_READABLE | G_PARAM_WRITABLE);
   g_object_class_install_property(gobject,
-				  PROP_INDEX,
+				  PROP_INPUT_LINES,
 				  param_spec);
-}
 
-void
-ags_play_dssi_audio_connectable_interface_init(AgsConnectableInterface *connectable)
-{
-  ags_play_dssi_audio_parent_connectable_interface = g_type_interface_peek_parent(connectable);
-
-  connectable->connect = ags_play_dssi_audio_connect;
-  connectable->disconnect = ags_play_dssi_audio_disconnect;
+  /**
+   * AgsPlayDssiAudio:output-lines:
+   *
+   * The effect's output lines count.
+   * 
+   * Since: 2.0.0
+   */
+  param_spec =  g_param_spec_ulong("output-lines",
+				   i18n_pspec("output lines of effect"),
+				   i18n_pspec("The effect's count of output lines"),
+				   0,
+				   65535,
+				   0,
+				   G_PARAM_READABLE | G_PARAM_WRITABLE);
+  g_object_class_install_property(gobject,
+				  PROP_OUTPUT_LINES,
+				  param_spec);
 }
 
 void
@@ -237,10 +216,6 @@ ags_play_dssi_audio_init(AgsPlayDssiAudio *play_dssi_audio)
   AGS_RECALL(play_dssi_audio)->build_id = AGS_RECALL_DEFAULT_BUILD_ID;
   AGS_RECALL(play_dssi_audio)->xml_type = "ags-play-dssi-audio";
 
-  play_dssi_audio->filename = NULL;
-  play_dssi_audio->effect = NULL;
-  play_dssi_audio->index = 0;
-  
   play_dssi_audio->bank = 0;
   play_dssi_audio->program = 0;
   
@@ -254,13 +229,6 @@ ags_play_dssi_audio_init(AgsPlayDssiAudio *play_dssi_audio)
   play_dssi_audio->output_lines = 0;
 }
 
-void
-ags_play_dssi_audio_finalize(GObject *gobject)
-{
-  /* call parent */
-  G_OBJECT_CLASS(ags_play_dssi_audio_parent_class)->finalize(gobject);
-}
-
 void
 ags_play_dssi_audio_set_property(GObject *gobject,
 				 guint prop_id,
@@ -269,51 +237,81 @@ ags_play_dssi_audio_set_property(GObject *gobject,
 {
   AgsPlayDssiAudio *play_dssi_audio;
 
+  pthread_mutex_t *recall_mutex;
+
   play_dssi_audio = AGS_PLAY_DSSI_AUDIO(gobject);
 
+  /* get recall mutex */
+  pthread_mutex_lock(ags_recall_get_class_mutex());
+  
+  recall_mutex = AGS_RECALL(gobject)->obj_mutex;
+  
+  pthread_mutex_unlock(ags_recall_get_class_mutex());
+
   switch(prop_id){
-  case PROP_FILENAME:
+  case PROP_PLUGIN:
     {
-      GObject *soundcard;
-      gchar *filename;
+      AgsDssiPlugin *dssi_plugin;
+      
+      dssi_plugin = g_value_get_object(value);
 
-      filename = g_value_get_string(value);
+      pthread_mutex_lock(recall_mutex);
 
-      if(filename == play_dssi_audio->filename){
+      if(play_dssi_audio->plugin == dssi_plugin){
+	pthread_mutex_unlock(recall_mutex);
+	
 	return;
       }
 
-      if(play_dssi_audio->filename != NULL){
-	g_free(play_dssi_audio->filename);
+      if(play_dssi_audio->plugin != NULL){
+	g_object_unref(play_dssi_audio->plugin);
       }
 
-      play_dssi_audio->filename = g_strdup(filename);
+      if(dssi_plugin != NULL){
+	g_object_ref(dssi_plugin);
+      }
+
+      play_dssi_audio->plugin = dssi_plugin;
+
+      pthread_mutex_unlock(recall_mutex);
     }
     break;
-  case PROP_EFFECT:
+  case PROP_INPUT_LINES:
     {
-      gchar *effect;
+      unsigned long effect_input_lines;
       
-      effect = g_value_get_string(value);
+      effect_input_lines = g_value_get_ulong(value);
+
+      pthread_mutex_lock(recall_mutex);
+
+      if(effect_input_lines == play_dssi_audio->input_lines){
+	pthread_mutex_unlock(recall_mutex);
 
-      if(effect == play_dssi_audio->effect){
 	return;
       }
 
-      play_dssi_audio->effect = g_strdup(effect);
+      play_dssi_audio->input_lines = effect_input_lines;
+
+      pthread_mutex_unlock(recall_mutex);
     }
     break;
-  case PROP_INDEX:
+  case PROP_OUTPUT_LINES:
     {
-      unsigned long index;
+      unsigned long effect_output_lines;
       
-      index = g_value_get_ulong(value);
+      effect_output_lines = g_value_get_ulong(value);
+
+      pthread_mutex_lock(recall_mutex);
+
+      if(effect_output_lines == play_dssi_audio->output_lines){
+	pthread_mutex_unlock(recall_mutex);
 
-      if(index == play_dssi_audio->index){
 	return;
       }
 
-      play_dssi_audio->index = index;
+      play_dssi_audio->output_lines = effect_output_lines;
+
+      pthread_mutex_unlock(recall_mutex);
     }
     break;
   default:
@@ -330,22 +328,43 @@ ags_play_dssi_audio_get_property(GObject *gobject,
 {
   AgsPlayDssiAudio *play_dssi_audio;
 
+  pthread_mutex_t *recall_mutex;
+
   play_dssi_audio = AGS_PLAY_DSSI_AUDIO(gobject);
 
+  /* get recall mutex */
+  pthread_mutex_lock(ags_recall_get_class_mutex());
+  
+  recall_mutex = AGS_RECALL(gobject)->obj_mutex;
+  
+  pthread_mutex_unlock(ags_recall_get_class_mutex());
+
   switch(prop_id){
-  case PROP_FILENAME:
+  case PROP_PLUGIN:
     {
-      g_value_set_string(value, play_dssi_audio->filename);
+      pthread_mutex_lock(recall_mutex);
+
+      g_value_set_object(value, play_dssi_audio->plugin);
+
+      pthread_mutex_unlock(recall_mutex);
     }
     break;
-  case PROP_EFFECT:
+  case PROP_INPUT_LINES:
     {
-      g_value_set_string(value, play_dssi_audio->effect);
+      pthread_mutex_lock(recall_mutex);
+
+      g_value_set_ulong(value, play_dssi_audio->input_lines);
+
+      pthread_mutex_unlock(recall_mutex);
     }
     break;
-  case PROP_INDEX:
+  case PROP_OUTPUT_LINES:
     {
-      g_value_set_uint(value, play_dssi_audio->index);
+      pthread_mutex_lock(recall_mutex);
+
+      g_value_set_ulong(value, play_dssi_audio->output_lines);
+
+      pthread_mutex_unlock(recall_mutex);
     }
     break;
   default:
@@ -354,35 +373,42 @@ ags_play_dssi_audio_get_property(GObject *gobject,
   }
 }
 
+void
+ags_play_dssi_audio_dispose(GObject *gobject)
+{
+  AgsPlayDssiAudio *play_dssi_audio;
+
+  play_dssi_audio = (AgsPlayDssiAudio *) gobject;
+
+  if(play_dssi_audio->plugin != NULL){
+    g_object_unref(play_dssi_audio->plugin);
+
+    play_dssi_audio->plugin = NULL;
+  }
+  
+  /* call parent */
+  G_OBJECT_CLASS(ags_play_dssi_audio_parent_class)->dispose(gobject);
+}
+
+void
+ags_play_dssi_audio_finalize(GObject *gobject)
+{
+  AgsPlayDssiAudio *play_dssi_audio;
+
+  play_dssi_audio = (AgsPlayDssiAudio *) gobject;
+
+  if(play_dssi_audio->plugin != NULL){
+    g_object_unref(play_dssi_audio->plugin);
+  }
+
+  /* call parent */
+  G_OBJECT_CLASS(ags_play_dssi_audio_parent_class)->finalize(gobject);
+}
+
 void
 ags_play_dssi_audio_set_ports(AgsPlugin *plugin, GList *port)
 {
-  while(port != NULL){
-    port = port->next;
-  }
-}
-
-void
-ags_play_dssi_audio_connect(AgsConnectable *connectable)
-{
-  AgsRecall *recall;
-
-  recall = AGS_RECALL(connectable);
-  
-  if((AGS_RECALL_CONNECTED & (recall->flags)) != 0){
-    return;
-  }
-
-  ags_recall_load_automation(recall,
-			     g_list_copy(recall->port));
-  
-  ags_play_dssi_audio_parent_connectable_interface->connect(connectable);
-}
-
-void
-ags_play_dssi_audio_disconnect(AgsConnectable *connectable)
-{
-  ags_play_dssi_audio_parent_connectable_interface->disconnect(connectable);
+  //TODO:JK: implement me
 }
 
 void
@@ -390,8 +416,10 @@ ags_play_dssi_audio_read(AgsFile *file, xmlNode *node, AgsPlugin *plugin)
 {
   AgsPlayDssiAudio *gobject;
   AgsDssiPlugin *dssi_plugin;
+
   gchar *filename, *effect;
-  unsigned long index;
+
+  unsigned long effect_index;
 
   gobject = AGS_PLAY_DSSI_AUDIO(plugin);
 
@@ -408,15 +436,15 @@ ags_play_dssi_audio_read(AgsFile *file, xmlNode *node, AgsPlugin *plugin)
 			"filename");
   effect = xmlGetProp(node,
 		      "effect");
-  index = g_ascii_strtoull(xmlGetProp(node,
-				      "index"),
-			   NULL,
-			   10);
-
+  effect_index = g_ascii_strtoull(xmlGetProp(node,
+					     "index"),
+				  NULL,
+				  10);
+  
   g_object_set(gobject,
 	       "filename", filename,
 	       "effect", effect,
-	       "index", index,
+	       "effect-index", effect_index,
 	       NULL);
 
   ags_play_dssi_audio_load(gobject);
@@ -426,7 +454,9 @@ xmlNode*
 ags_play_dssi_audio_write(AgsFile *file, xmlNode *parent, AgsPlugin *plugin)
 {
   AgsPlayDssiAudio *play_dssi_audio;
+
   xmlNode *node;
+
   gchar *id;
 
   play_dssi_audio = AGS_PLAY_DSSI_AUDIO(plugin);
@@ -450,15 +480,15 @@ ags_play_dssi_audio_write(AgsFile *file, xmlNode *parent, AgsPlugin *plugin)
 
   xmlNewProp(node,
 	     "filename",
-	     g_strdup(play_dssi_audio->filename));
+	     g_strdup(AGS_RECALL(play_dssi_audio)->filename));
 
   xmlNewProp(node,
 	     "effect",
-	     g_strdup(play_dssi_audio->effect));
+	     g_strdup(AGS_RECALL(play_dssi_audio)->effect));
 
   xmlNewProp(node,
 	     "index",
-	     g_strdup_printf("%lu", play_dssi_audio->index));
+	     g_strdup_printf("%lu", AGS_RECALL(play_dssi_audio)->effect_index));
 
   xmlAddChild(parent,
 	      node);
@@ -468,113 +498,233 @@ ags_play_dssi_audio_write(AgsFile *file, xmlNode *parent, AgsPlugin *plugin)
 
 /**
  * ags_play_dssi_audio_load:
- * @play_dssi_audio: an #AgsPlayDssiAudio
+ * @play_dssi_audio: the #AgsPlayDssiAudio
  *
  * Set up DSSI handle.
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_play_dssi_audio_load(AgsPlayDssiAudio *play_dssi_audio)
 {
   AgsDssiPlugin *dssi_plugin;
 
+  gchar *filename, *effect;
+
+  unsigned long effect_index;
+  
   void *plugin_so;
   DSSI_Descriptor_Function dssi_descriptor;
   DSSI_Descriptor *plugin_descriptor;
-
-  /*  */
-  play_dssi_audio->plugin = 
-    dssi_plugin = ags_dssi_manager_find_dssi_plugin(ags_dssi_manager_get_instance(),
-						    play_dssi_audio->filename, play_dssi_audio->effect);
   
-  plugin_so = AGS_BASE_PLUGIN(dssi_plugin)->plugin_so;
+  pthread_mutex_t *recall_mutex;
 
+  if(!AGS_IS_PLAY_DSSI_AUDIO(play_dssi_audio)){
+    return;
+  }
+  
+  /* get recall mutex */
+  pthread_mutex_lock(ags_recall_get_class_mutex());
+  
+  recall_mutex = AGS_RECALL(play_dssi_audio)->obj_mutex;
+  
+  pthread_mutex_unlock(ags_recall_get_class_mutex());
+
+  /* get filename and effect */
+  pthread_mutex_lock(recall_mutex);
+
+  filename = g_strdup(AGS_RECALL(play_dssi_audio)->filename);
+  effect = g_strdup(AGS_RECALL(play_dssi_audio)->effect);
+  
+  pthread_mutex_unlock(recall_mutex);
+  
+  /* find AgsDssiPlugin */
+  dssi_plugin = ags_dssi_manager_find_dssi_plugin(ags_dssi_manager_get_instance(),
+						  filename, effect);
+
+  /* get some fields */
+  g_object_get(play_dssi_audio,
+	       "effect-index", &effect_index,
+	       "plugin", &dssi_plugin,
+	       NULL);
+
+  g_object_get(dssi_plugin,
+	       "plugin-so", &plugin_so,
+	       NULL);
+
+  /* dssi descriptor function - dlsym */
   if(plugin_so){
     dssi_descriptor = (DSSI_Descriptor_Function) dlsym(plugin_so,
 						       "dssi_descriptor");
 
     if(dlerror() == NULL && dssi_descriptor){
+      pthread_mutex_lock(recall_mutex);
+      
       play_dssi_audio->plugin_descriptor = 
-	plugin_descriptor = dssi_descriptor(play_dssi_audio->index);
+	plugin_descriptor = dssi_descriptor(effect_index);
+
+      pthread_mutex_unlock(recall_mutex);
     }
   }
+
+  /* free */
+  g_free(filename);
+  g_free(effect);
 }
 
 /**
  * ags_play_dssi_audio_load_ports:
- * @play_dssi_audio: an #AgsPlayDssiAudio
+ * @play_dssi_audio: the #AgsPlayDssiAudio
  *
  * Set up DSSI ports.
  *
- * Returns: a #GList containing #AgsPort.
+ * Returns: a #GList-struct containing #AgsPort.
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 GList*
 ags_play_dssi_audio_load_ports(AgsPlayDssiAudio *play_dssi_audio)
 {
-  AgsDssiPlugin *dssi_plugin;
   AgsPort *current;
 
-  GList *port;
-  GList *port_descriptor;
+  AgsDssiPlugin *dssi_plugin;
+
+  GList *start_port;
+  GList *start_plugin_port, *plugin_port;
+
+  gchar *filename, *effect;
+  gchar *plugin_name;
 
   unsigned long port_count;
   unsigned long i;
 
-  dssi_plugin = ags_dssi_manager_find_dssi_plugin(ags_dssi_manager_get_instance(),
-						  play_dssi_audio->filename, play_dssi_audio->effect);
+  pthread_mutex_t *recall_mutex;
+  pthread_mutex_t *base_plugin_mutex;
 
-  port = NULL;
-  port_descriptor = AGS_BASE_PLUGIN(dssi_plugin)->port;
+  if(!AGS_IS_PLAY_DSSI_AUDIO(play_dssi_audio)){
+    return(NULL);
+  }
+
+  /* get recall mutex */
+  pthread_mutex_lock(ags_recall_get_class_mutex());
   
-  if(port_descriptor != NULL){
-    port_count = g_list_length(port_descriptor);
+  recall_mutex = AGS_RECALL(play_dssi_audio)->obj_mutex;
+  
+  pthread_mutex_unlock(ags_recall_get_class_mutex());
+
+  /* get filename and effect */
+  pthread_mutex_lock(recall_mutex);
+
+  filename = g_strdup(AGS_RECALL(play_dssi_audio)->filename);
+  effect = g_strdup(AGS_RECALL(play_dssi_audio)->effect);
+  
+  pthread_mutex_unlock(recall_mutex);
+  
+  /* find AgsDssiPlugin */
+  dssi_plugin = ags_dssi_manager_find_dssi_plugin(ags_dssi_manager_get_instance(),
+						  filename, effect);
+
+  /* get plugin port */
+  g_object_get(dssi_plugin,
+	       "plugin-port", &start_plugin_port,
+	       NULL);
+
+  /* get base plugin mutex */
+  pthread_mutex_lock(ags_base_plugin_get_class_mutex());
+  
+  base_plugin_mutex = AGS_BASE_PLUGIN(dssi_plugin)->obj_mutex;
+  
+  pthread_mutex_unlock(ags_base_plugin_get_class_mutex());
+  
+  /* get plugin name */
+  pthread_mutex_lock(base_plugin_mutex);
+
+  plugin_name = g_strdup_printf("dssi-%lu",
+				dssi_plugin->unique_id);
+
+  pthread_mutex_unlock(base_plugin_mutex);
+  
+  plugin_port = start_plugin_port;
+  start_port = NULL;
+  
+  if(plugin_port != NULL){
+    port_count = g_list_length(start_plugin_port);
     
     for(i = 0; i < port_count; i++){
-      if((AGS_PORT_DESCRIPTOR_CONTROL & (AGS_PORT_DESCRIPTOR(port_descriptor->data)->flags)) != 0){
-	gchar *plugin_name;
+      if(ags_plugin_port_test_flags(plugin_port->data, AGS_PLUGIN_PORT_CONTROL)){
 	gchar *specifier;
-	
-	plugin_name = g_strdup_printf("dssi-%lu", dssi_plugin->unique_id);
-	specifier = AGS_PORT_DESCRIPTOR(port_descriptor->data)->port_name;
 
+	GValue value = {0,};
+	
+	pthread_mutex_t *plugin_port_mutex;
+
+	/* get plugin port mutex */
+	pthread_mutex_lock(ags_plugin_port_get_class_mutex());
+	
+	plugin_port_mutex = AGS_PLUGIN_PORT(plugin_port->data)->obj_mutex;
+	
+	pthread_mutex_unlock(ags_plugin_port_get_class_mutex());
+
+	/* get specifier */
+	pthread_mutex_lock(plugin_port_mutex);
+	
+	specifier = g_strdup(AGS_PLUGIN_PORT(plugin_port->data)->port_name);
+
+	pthread_mutex_unlock(plugin_port_mutex);
+
+	if(specifier == NULL){
+	  plugin_port = plugin_port->next;
+	  
+	  continue;
+	}
+
+	/* new port */
 	current = g_object_new(AGS_TYPE_PORT,
 			       "plugin-name", plugin_name,
 			       "specifier", specifier,
 			       "control-port", g_strdup_printf("%lu/%lu",
-								 i,
+								 i + 1,
 								 port_count),
 			       "port-value-is-pointer", FALSE,
 			       "port-value-type", G_TYPE_FLOAT,
 			       NULL);
-	g_object_ref(current);
 	current->flags |= AGS_PORT_USE_LADSPA_FLOAT;
+	g_object_ref(current);
 
-	if((AGS_PORT_DESCRIPTOR_OUTPUT & (AGS_PORT_DESCRIPTOR(port_descriptor->data)->flags)) != 0){
-	  AGS_RECALL(play_dssi_audio)->flags |= AGS_RECALL_HAS_OUTPUT_PORT;
+	if(ags_plugin_port_test_flags(plugin_port->data, AGS_PLUGIN_PORT_OUTPUT)){
+	  ags_recall_set_flags(play_dssi_audio,
+			       AGS_RECALL_HAS_OUTPUT_PORT);
 
 	  current->flags |= AGS_PORT_IS_OUTPUT;
 	}else{
-	  if((AGS_PORT_DESCRIPTOR_INTEGER & (AGS_PORT_DESCRIPTOR(port_descriptor->data)->flags)) == 0 &&
-	     (AGS_PORT_DESCRIPTOR_TOGGLED & (AGS_PORT_DESCRIPTOR(port_descriptor->data)->flags)) == 0){
+	  if(!ags_plugin_port_test_flags(plugin_port->data, AGS_PLUGIN_PORT_INTEGER) &&
+	     !ags_plugin_port_test_flags(plugin_port->data, AGS_PLUGIN_PORT_TOGGLED)){
 	    current->flags |= AGS_PORT_INFINITE_RANGE;
 	  }
 	}
 	
-	current->port_descriptor = port_descriptor->data;
+	g_object_set(current,
+		     "plugin-port", plugin_port->data,
+		     NULL);
+	
 	ags_play_dssi_audio_load_conversion(play_dssi_audio,
 					    (GObject *) current,
-					    port_descriptor->data);
+					    (GObject *) plugin_port->data);
+
+	g_object_get_property(plugin_port->data,
+			      "default-value",
+			      &value);
 	
-	current->port_value.ags_port_ladspa = g_value_get_float(AGS_PORT_DESCRIPTOR(port_descriptor->data)->default_value);
+	current->port_value.ags_port_ladspa = g_value_get_float(&value);
 	//	g_message("connecting port: %d/%d %f", i, port_count, current->port_value.ags_port_float);
 
-	port = g_list_prepend(port,
-			      current);
-      }else if((AGS_PORT_DESCRIPTOR_AUDIO & (AGS_PORT_DESCRIPTOR(port_descriptor->data)->flags)) != 0){
-	if((AGS_PORT_DESCRIPTOR_INPUT & (AGS_PORT_DESCRIPTOR(port_descriptor->data)->flags)) != 0){
+	start_port = g_list_prepend(start_port,
+				    current);
+      }else if(ags_plugin_port_test_flags(plugin_port->data, AGS_PLUGIN_PORT_AUDIO)){
+	/* audio port */
+	if(ags_plugin_port_test_flags(plugin_port->data, AGS_PLUGIN_PORT_INPUT)){
+	  pthread_mutex_lock(recall_mutex);
+	  
 	  if(play_dssi_audio->input_port == NULL){
 	    play_dssi_audio->input_port = (unsigned long *) malloc(sizeof(unsigned long));
 	    play_dssi_audio->input_port[0] = i;
@@ -585,7 +735,11 @@ ags_play_dssi_audio_load_ports(AgsPlayDssiAudio *play_dssi_audio)
 	  }
 	  
 	  play_dssi_audio->input_lines += 1;
-	}else if((AGS_PORT_DESCRIPTOR_OUTPUT & (AGS_PORT_DESCRIPTOR(port_descriptor->data)->flags)) != 0){
+
+	  pthread_mutex_unlock(recall_mutex);
+	}else if(ags_plugin_port_test_flags(plugin_port->data, AGS_PLUGIN_PORT_OUTPUT)){
+	  pthread_mutex_lock(recall_mutex);
+	  
 	  if(play_dssi_audio->output_port == NULL){
 	    play_dssi_audio->output_port = (unsigned long *) malloc(sizeof(unsigned long));
 	    play_dssi_audio->output_port[0] = i;
@@ -596,46 +750,58 @@ ags_play_dssi_audio_load_ports(AgsPlayDssiAudio *play_dssi_audio)
 	  }
 	  
 	  play_dssi_audio->output_lines += 1;
+
+	  pthread_mutex_unlock(recall_mutex);	  
 	}
       }
 
-      port_descriptor = port_descriptor->next;
+      /* iterate */
+      plugin_port = plugin_port->next;
     }
-    
-    AGS_RECALL(play_dssi_audio)->port = g_list_reverse(port);
+
+    start_port = g_list_reverse(start_port);
+    AGS_RECALL(play_dssi_audio)->port = g_list_copy(start_port);
   }
 
-  g_message("output lines: %d", play_dssi_audio->output_lines);
+  /* free */
+  g_free(filename);
+  g_free(effect);
 
-  return(g_list_copy(AGS_RECALL(play_dssi_audio)->port));
+  g_free(plugin_name);
+
+#ifdef AGS_DEBUG
+  g_message("output lines: %d", play_dssi_audio->output_lines);
+#endif
+  
+  return(start_port);
 }
 
 /**
  * ags_play_dssi_audio_load_conversion:
  * @play_dssi_audio: the #AgsPlayDssiAudio
- * @port: an #AgsPort
- * @port_descriptor: the #AgsPortDescriptor-struct
+ * @port: the #AgsPort
+ * @plugin_port: the #AgsPluginPort
  * 
- * Loads conversion object by using @port_descriptor and sets in on @port.
+ * Loads conversion object by using @plugin_port and sets in on @port.
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_play_dssi_audio_load_conversion(AgsPlayDssiAudio *play_dssi_audio,
 				    GObject *port,
-				    gpointer port_descriptor)
+				    GObject *plugin_port)
 {
   AgsLadspaConversion *ladspa_conversion;
 
-  if(play_dssi_audio == NULL ||
-     port == NULL ||
-     port_descriptor == NULL){
+  if(!AGS_IS_PLAY_DSSI_AUDIO(play_dssi_audio) ||
+     !AGS_IS_PORT(port) ||
+     !AGS_PLUGIN_PORT(plugin_port)){
     return;
   }
 
   ladspa_conversion = NULL;
 
-  if((AGS_PORT_DESCRIPTOR_BOUNDED_BELOW & (AGS_PORT_DESCRIPTOR(port_descriptor)->flags)) != 0){
+  if(ags_plugin_port_test_flags(plugin_port, AGS_PLUGIN_PORT_BOUNDED_BELOW)){
     if(ladspa_conversion == NULL ||
        !AGS_IS_LADSPA_CONVERSION(ladspa_conversion)){
       ladspa_conversion = ags_ladspa_conversion_new();
@@ -644,7 +810,7 @@ ags_play_dssi_audio_load_conversion(AgsPlayDssiAudio *play_dssi_audio,
     ladspa_conversion->flags |= AGS_LADSPA_CONVERSION_BOUNDED_BELOW;
   }
 
-  if((AGS_PORT_DESCRIPTOR_BOUNDED_ABOVE & (AGS_PORT_DESCRIPTOR(port_descriptor)->flags)) != 0){
+  if(ags_plugin_port_test_flags(plugin_port, AGS_PLUGIN_PORT_BOUNDED_ABOVE)){
     if(ladspa_conversion == NULL ||
        !AGS_IS_LADSPA_CONVERSION(ladspa_conversion)){
       ladspa_conversion = ags_ladspa_conversion_new();
@@ -653,7 +819,7 @@ ags_play_dssi_audio_load_conversion(AgsPlayDssiAudio *play_dssi_audio,
     ladspa_conversion->flags |= AGS_LADSPA_CONVERSION_BOUNDED_ABOVE;
   }
   
-  if((AGS_PORT_DESCRIPTOR_SAMPLERATE & (AGS_PORT_DESCRIPTOR(port_descriptor)->flags)) != 0){
+  if(ags_plugin_port_test_flags(plugin_port, AGS_PLUGIN_PORT_SAMPLERATE)){
     if(ladspa_conversion == NULL ||
        !AGS_IS_LADSPA_CONVERSION(ladspa_conversion)){
       ladspa_conversion = ags_ladspa_conversion_new();
@@ -662,7 +828,7 @@ ags_play_dssi_audio_load_conversion(AgsPlayDssiAudio *play_dssi_audio,
     ladspa_conversion->flags |= AGS_LADSPA_CONVERSION_SAMPLERATE;
   }
 
-  if((AGS_PORT_DESCRIPTOR_LOGARITHMIC & (AGS_PORT_DESCRIPTOR(port_descriptor)->flags)) != 0){
+  if(ags_plugin_port_test_flags(plugin_port, AGS_PLUGIN_PORT_LOGARITHMIC)){
     if(ladspa_conversion == NULL ||
        !AGS_IS_LADSPA_CONVERSION(ladspa_conversion)){
       ladspa_conversion = ags_ladspa_conversion_new();
@@ -680,15 +846,15 @@ ags_play_dssi_audio_load_conversion(AgsPlayDssiAudio *play_dssi_audio,
 
 /**
  * ags_play_dssi_audio_find:
- * @recall: a #GList containing #AgsRecall
- * @filename: plugin filename
- * @effect: effect's name
+ * @recall: a #GList-struct containing #AgsRecall
+ * @filename: plugin's filename
+ * @effect: plugin's effect
  *
- * Retrieve DSSI recall.
+ * Find DSSI recall.
  *
- * Returns: Next match.
+ * Returns: Next matching #GList-struct, or %NULL if not found
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 GList*
 ags_play_dssi_audio_find(GList *recall,
@@ -696,35 +862,56 @@ ags_play_dssi_audio_find(GList *recall,
 {
   while(recall != NULL){
     if(AGS_IS_PLAY_DSSI_AUDIO(recall->data)){
-      if(!g_strcmp0(AGS_PLAY_DSSI_AUDIO(recall->data)->filename,
-		    filename) &&
-	 !g_strcmp0(AGS_PLAY_DSSI_AUDIO(recall->data)->effect,
-		    effect)){
-	return(recall);
+      gboolean success;
+      
+      pthread_mutex_t *recall_mutex;
+
+      /* get recall mutex */
+      pthread_mutex_lock(ags_recall_get_class_mutex());
+  
+      recall_mutex = AGS_RECALL(recall->data)->obj_mutex;
+  
+      pthread_mutex_unlock(ags_recall_get_class_mutex());
+
+      /* check current filename and effect */
+      pthread_mutex_lock(recall_mutex);
+
+      success = (!g_strcmp0(AGS_RECALL(recall->data)->filename,
+			    filename) &&
+		 !g_strcmp0(AGS_RECALL(recall->data)->effect,
+			    effect)) ? TRUE: FALSE;
+      
+      pthread_mutex_unlock(recall_mutex);
+
+      if(success){
+	break;
       }
     }
 
+    /* iterate */
     recall = recall->next;
   }
 
-  return(NULL);
+  return(recall);
 }
 
 /**
  * ags_play_dssi_audio_new:
+ * @audio: the #AgsAudio
  *
- * Creates an #AgsPlayDssiAudio
+ * Create a new instance of #AgsPlayDssiAudio
  *
- * Returns: a new #AgsPlayDssiAudio
+ * Returns: the new #AgsPlayDssiAudio
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsPlayDssiAudio*
-ags_play_dssi_audio_new()
+ags_play_dssi_audio_new(AgsAudio *audio)
 {
   AgsPlayDssiAudio *play_dssi_audio;
 
   play_dssi_audio = (AgsPlayDssiAudio *) g_object_new(AGS_TYPE_PLAY_DSSI_AUDIO,
+						      "audio", audio,
 						      NULL);
 
   return(play_dssi_audio);
