@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2017 Joël Krähemann
+ * Copyright (C) 2005-2018 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -19,12 +19,13 @@
 
 #include <ags/audio/task/ags_move_note.h>
 
+#include <ags/audio/ags_audio.h>
+
 #include <ags/i18n.h>
 
 #include <math.h>
 
 void ags_move_note_class_init(AgsMoveNoteClass *move_note);
-void ags_move_note_connectable_interface_init(AgsConnectableInterface *connectable);
 void ags_move_note_init(AgsMoveNote *move_note);
 void ags_move_note_set_property(GObject *gobject,
 				guint prop_id,
@@ -34,8 +35,6 @@ void ags_move_note_get_property(GObject *gobject,
 				guint prop_id,
 				GValue *value,
 				GParamSpec *param_spec);
-void ags_move_note_connect(AgsConnectable *connectable);
-void ags_move_note_disconnect(AgsConnectable *connectable);
 void ags_move_note_dispose(GObject *gobject);
 void ags_move_note_finalize(GObject *gobject);
 
@@ -52,11 +51,9 @@ void ags_move_note_launch(AgsTask *task);
  */
 
 static gpointer ags_move_note_parent_class = NULL;
-static AgsConnectableInterface *ags_move_note_parent_connectable_interface;
 
 enum{
   PROP_0,
-  PROP_AUDIO,
   PROP_NOTATION,
   PROP_SELECTION,
   PROP_FIRST_X,
@@ -73,36 +70,26 @@ ags_move_note_get_type()
   static volatile gsize g_define_type_id__volatile = 0;
 
   if(g_once_init_enter (&g_define_type_id__volatile)){
-    GType ags_type_move_note;
+    GType ags_type_move_note = 0;
 
     static const GTypeInfo ags_move_note_info = {
-      sizeof (AgsMoveNoteClass),
+      sizeof(AgsMoveNoteClass),
       NULL, /* base_init */
       NULL, /* base_finalize */
       (GClassInitFunc) ags_move_note_class_init,
       NULL, /* class_finalize */
       NULL, /* class_data */
-      sizeof (AgsMoveNote),
+      sizeof(AgsMoveNote),
       0,    /* n_preallocs */
       (GInstanceInitFunc) ags_move_note_init,
     };
 
-    static const GInterfaceInfo ags_connectable_interface_info = {
-      (GInterfaceInitFunc) ags_move_note_connectable_interface_init,
-      NULL, /* interface_finalize */
-      NULL, /* interface_data */
-    };
-
     ags_type_move_note = g_type_register_static(AGS_TYPE_TASK,
 						"AgsMoveNote",
 						&ags_move_note_info,
 						0);
 
-    g_type_add_interface_static(ags_type_move_note,
-				AGS_TYPE_CONNECTABLE,
-				&ags_connectable_interface_info);
-
-    g_once_init_leave (&g_define_type_id__volatile, ags_type_move_note);
+    g_once_init_leave(&g_define_type_id__volatile, ags_type_move_note);
   }
 
   return g_define_type_id__volatile;
@@ -113,6 +100,7 @@ ags_move_note_class_init(AgsMoveNoteClass *move_note)
 {
   GObjectClass *gobject;
   AgsTaskClass *task;
+
   GParamSpec *param_spec;
 
   ags_move_note_parent_class = g_type_class_peek_parent(move_note);
@@ -127,28 +115,12 @@ ags_move_note_class_init(AgsMoveNoteClass *move_note)
   gobject->finalize = ags_move_note_finalize;
 
   /* properties */
-  /**
-   * AgsMoveNote:audio:
-   *
-   * The assigned #AgsAudio
-   * 
-   * Since: 1.2.2
-   */
-  param_spec = g_param_spec_object("audio",
-				   i18n_pspec("audio of move note"),
-				   i18n_pspec("The audio of move note task"),
-				   AGS_TYPE_AUDIO,
-				   G_PARAM_READABLE | G_PARAM_WRITABLE);
-  g_object_class_install_property(gobject,
-				  PROP_AUDIO,
-				  param_spec);
-
   /**
    * AgsMoveNote:notation:
    *
    * The assigned #AgsNotation
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_object("notation",
 				   i18n_pspec("notation of move note"),
@@ -164,7 +136,7 @@ ags_move_note_class_init(AgsMoveNoteClass *move_note)
    *
    * The assigned #AgsNote
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_pointer("selection",
 				    i18n_pspec("selection to move"),
@@ -179,7 +151,7 @@ ags_move_note_class_init(AgsMoveNoteClass *move_note)
    *
    * Move notation from x offset.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec =  g_param_spec_uint("first-x",
 				  i18n_pspec("move from x offset"),
@@ -197,7 +169,7 @@ ags_move_note_class_init(AgsMoveNoteClass *move_note)
    *
    * Move notation with x padding.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec =  g_param_spec_uint("first-y",
 				  i18n_pspec("move with x padding"),
@@ -215,7 +187,7 @@ ags_move_note_class_init(AgsMoveNoteClass *move_note)
    *
    * Move notation by move-x amount.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_int("move-x",
 				i18n_pspec("move with move-x amount"),
@@ -233,7 +205,7 @@ ags_move_note_class_init(AgsMoveNoteClass *move_note)
    *
    * Move notation by move-y amount.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec =  g_param_spec_int("move-y",
 				 i18n_pspec("move with move-y amount"),
@@ -251,7 +223,7 @@ ags_move_note_class_init(AgsMoveNoteClass *move_note)
    *
    * Move notation by relative position.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec =  g_param_spec_boolean("relative",
 				     i18n_pspec("move relative"),
@@ -267,7 +239,7 @@ ags_move_note_class_init(AgsMoveNoteClass *move_note)
    *
    * Move notation by absolute position.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec =  g_param_spec_boolean("absolute",
 				     i18n_pspec("move absolute"),
@@ -284,19 +256,9 @@ ags_move_note_class_init(AgsMoveNoteClass *move_note)
   task->launch = ags_move_note_launch;
 }
 
-void
-ags_move_note_connectable_interface_init(AgsConnectableInterface *connectable)
-{
-  ags_move_note_parent_connectable_interface = g_type_interface_peek_parent(connectable);
-
-  connectable->connect = ags_move_note_connect;
-  connectable->disconnect = ags_move_note_disconnect;
-}
-
 void
 ags_move_note_init(AgsMoveNote *move_note)
 {
-  move_note->audio = NULL;
   move_note->notation = NULL;
 
   move_note->selection = NULL;
@@ -321,27 +283,6 @@ ags_move_note_set_property(GObject *gobject,
   move_note = AGS_MOVE_NOTE(gobject);
 
   switch(prop_id){
-  case PROP_AUDIO:
-    {
-      AgsAudio *audio;
-
-      audio = (AgsAudio *) g_value_get_object(value);
-
-      if(move_note->audio == (GObject *) audio){
-	return;
-      }
-
-      if(move_note->audio != NULL){
-	g_object_unref(move_note->audio);
-      }
-
-      if(audio != NULL){
-	g_object_ref(audio);
-      }
-
-      move_note->audio = (GObject *) audio;
-    }
-    break;
   case PROP_NOTATION:
     {
       AgsNotation *notation;
@@ -427,11 +368,6 @@ ags_move_note_get_property(GObject *gobject,
   move_note = AGS_MOVE_NOTE(gobject);
 
   switch(prop_id){
-  case PROP_AUDIO:
-    {
-      g_value_set_object(value, move_note->audio);
-    }
-    break;
   case PROP_NOTATION:
     {
       g_value_set_object(value, move_note->notation);
@@ -479,22 +415,6 @@ ags_move_note_get_property(GObject *gobject,
   }
 }
 
-void
-ags_move_note_connect(AgsConnectable *connectable)
-{
-  ags_move_note_parent_connectable_interface->connect(connectable);
-
-  /* empty */
-}
-
-void
-ags_move_note_disconnect(AgsConnectable *connectable)
-{
-  ags_move_note_parent_connectable_interface->disconnect(connectable);
-
-  /* empty */
-}
-
 void
 ags_move_note_dispose(GObject *gobject)
 {
@@ -502,12 +422,6 @@ ags_move_note_dispose(GObject *gobject)
 
   move_note = AGS_MOVE_NOTE(gobject);
 
-  if(move_note->audio != NULL){
-    g_object_unref(move_note->audio);
-
-    move_note->audio = NULL;
-  }
-
   if(move_note->notation != NULL){
     g_object_unref(move_note->notation);
 
@@ -531,10 +445,6 @@ ags_move_note_finalize(GObject *gobject)
 
   move_note = AGS_MOVE_NOTE(gobject);
 
-  if(move_note->audio != NULL){
-    g_object_unref(move_note->audio);
-  }
-
   if(move_note->notation != NULL){
     g_object_unref(move_note->notation);
   }
@@ -556,10 +466,9 @@ ags_move_note_launch(AgsTask *task)
   
   AgsMoveNote *move_note;
 
-  AgsMutexManager *mutex_manager;
-  
   GList *selection;
 
+  guint audio_channel;
   guint first_x;
   guint first_y;
   gint move_x;
@@ -568,20 +477,17 @@ ags_move_note_launch(AgsTask *task)
   gboolean relative;
   gboolean absolute;
 
-  pthread_mutex_t *application_mutex;
-  pthread_mutex_t *audio_mutex;
-
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-
   move_note = AGS_MOVE_NOTE(task);
 
   /* get some properties */
-  audio = move_note->audio;
-
   notation =
     current_notation = move_note->notation;
 
+  g_object_get(notation,
+	       "audio", &audio,
+	       "audio-channel", &audio_channel,
+	       NULL);
+
   selection = move_note->selection;
 
   first_x = move_note->first_x;
@@ -593,17 +499,7 @@ ags_move_note_launch(AgsTask *task)
   relative = move_note->relative;
   absolute = move_note->absolute;
 
-  /* get audio mutex */
-  pthread_mutex_lock(application_mutex);
-
-  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
-					 (GObject *) audio);
-  
-  pthread_mutex_unlock(application_mutex);
-
   /* move */
-  pthread_mutex_lock(audio_mutex);
-
   while(selection != NULL){
     note = ags_note_duplicate(AGS_NOTE(selection->data));
 
@@ -619,25 +515,32 @@ ags_move_note_launch(AgsTask *task)
       note->y = move_y + (note->y + first_y);
     }
 
-    if(note->x[0] >= current_notation->timestamp->timer.ags_offset.offset + AGS_NOTATION_DEFAULT_OFFSET){
+    if(note->x[0] >= ags_timestamp_get_ags_offset(current_notation->timestamp) + AGS_NOTATION_DEFAULT_OFFSET){
       AgsTimestamp *timestamp;
 
+      GList *list_start;
+
+      g_object_get(audio,
+		   "notation", &list_start,
+		   NULL);
+      
       timestamp = ags_timestamp_new();
       timestamp->flags &= (~AGS_TIMESTAMP_UNIX);
       timestamp->flags |= AGS_TIMESTAMP_OFFSET;
       
       timestamp->timer.ags_offset.offset = (guint64) (AGS_NOTATION_DEFAULT_OFFSET * floor(note->x[0] / AGS_NOTATION_DEFAULT_OFFSET));
 
-      if((current_notation = ags_notation_find_near_timestamp(move_note->audio->notation, notation->audio_channel,
+      if((current_notation = ags_notation_find_near_timestamp(list_start, audio_channel,
 							      timestamp)) == NULL){
-	current_notation = ags_notation_new(notation->audio,
-					    notation->audio_channel);
+	current_notation = ags_notation_new(audio,
+					    audio_channel);
 	
-	current_notation->timestamp->timer.ags_offset.offset = timestamp->timer.ags_offset.offset;
-	move_note->audio->notation = ags_notation_add(move_note->audio->notation,
-						      current_notation);
+	current_notation->timestamp->timer.ags_offset.offset = ags_timestamp_get_ags_offset(timestamp);
+	ags_audio_add_notation(audio,
+			       current_notation);
       }
 
+      g_list_free(list_start);
       g_object_unref(timestamp);
     }
 
@@ -656,8 +559,6 @@ ags_move_note_launch(AgsTask *task)
 
     selection = selection->next;
   }
-
-  pthread_mutex_unlock(audio_mutex);
 }
 
 /**
@@ -671,13 +572,12 @@ ags_move_note_launch(AgsTask *task)
  * @relative: if %TRUE move relative position
  * @absolute: if %TRUE move absolute position
  *
- * WARNING you need to provide #AgsAudio as a property.
- * Creates an #AgsMoveNote task. Note either @relative or @absolute shall
+ * Create a new instance of #AgsMoveNote task. Note either @relative or @absolute shall
  * be %TRUE else it won't have any effect.
  *
  * Returns: a new #AgsMoveNote
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsMoveNote*
 ags_move_note_new(AgsNotation *notation,
