@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2015 Joël Krähemann
+ * Copyright (C) 2005-2018 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -19,15 +19,10 @@
 
 #include <ags/plugin/ags_lv2_urid_manager.h>
 
-#include <ags/libags.h>
-
-#include <pthread.h>
+#include <ags/object/ags_connectable.h>
 
 void ags_lv2_urid_manager_class_init(AgsLv2UridManagerClass *lv2_urid_manager);
-void ags_lv2_urid_manager_connectable_interface_init(AgsConnectableInterface *connectable);
 void ags_lv2_urid_manager_init(AgsLv2UridManager *lv2_urid_manager);
-void ags_lv2_urid_manager_connect(AgsConnectable *connectable);
-void ags_lv2_urid_manager_disconnect(AgsConnectable *connectable);
 void ags_lv2_urid_manager_finalize(GObject *gobject);
 
 void ags_lv2_urid_manager_destroy_data(gpointer data);
@@ -46,6 +41,8 @@ gboolean ags_lv2_urid_manager_finder(gpointer key, gpointer value, gpointer user
 
 static gpointer ags_lv2_urid_manager_parent_class = NULL;
 
+static pthread_mutex_t ags_lv2_urid_manager_class_mutex = PTHREAD_MUTEX_INITIALIZER;
+
 AgsLv2UridManager *ags_lv2_urid_manager = NULL;
 
 GType
@@ -54,36 +51,26 @@ ags_lv2_urid_manager_get_type()
   static volatile gsize g_define_type_id__volatile = 0;
 
   if(g_once_init_enter (&g_define_type_id__volatile)){
-    GType ags_type_lv2_urid_manager;
+    GType ags_type_lv2_urid_manager = 0;
 
     const GTypeInfo ags_lv2_urid_manager_info = {
-      sizeof(AgsLv2UridManagerClass),
+      sizeof (AgsLv2UridManagerClass),
       NULL, /* base_init */
       NULL, /* base_finalize */
       (GClassInitFunc) ags_lv2_urid_manager_class_init,
       NULL, /* class_finalize */
       NULL, /* class_data */
-      sizeof(AgsLv2UridManager),
+      sizeof (AgsLv2UridManager),
       0,    /* n_preallocs */
       (GInstanceInitFunc) ags_lv2_urid_manager_init,
     };
 
-    const GInterfaceInfo ags_connectable_interface_info = {
-      (GInterfaceInitFunc) ags_lv2_urid_manager_connectable_interface_init,
-      NULL, /* interface_finalize */
-      NULL, /* interface_data */
-    };
-
     ags_type_lv2_urid_manager = g_type_register_static(G_TYPE_OBJECT,
 						       "AgsLv2UridManager",
 						       &ags_lv2_urid_manager_info,
 						       0);
 
-    g_type_add_interface_static(ags_type_lv2_urid_manager,
-				AGS_TYPE_CONNECTABLE,
-				&ags_connectable_interface_info);
-
-    g_once_init_leave (&g_define_type_id__volatile, ags_type_lv2_urid_manager);
+    g_once_init_leave(&g_define_type_id__volatile, ags_type_lv2_urid_manager);
   }
 
   return g_define_type_id__volatile;
@@ -103,16 +90,24 @@ ags_lv2_urid_manager_class_init(AgsLv2UridManagerClass *lv2_urid_manager)
   gobject->finalize = ags_lv2_urid_manager_finalize;
 }
 
-void
-ags_lv2_urid_manager_connectable_interface_init(AgsConnectableInterface *connectable)
-{
-  connectable->connect = ags_lv2_urid_manager_connect;
-  connectable->disconnect = ags_lv2_urid_manager_disconnect;
-}
-
 void
 ags_lv2_urid_manager_init(AgsLv2UridManager *lv2_urid_manager)
 {
+  /* lv2 urid manager mutex */
+  lv2_urid_manager->obj_mutexattr = (pthread_mutexattr_t *) malloc(sizeof(pthread_mutexattr_t));
+  pthread_mutexattr_init(lv2_urid_manager->obj_mutexattr);
+  pthread_mutexattr_settype(lv2_urid_manager->obj_mutexattr,
+			    PTHREAD_MUTEX_RECURSIVE);
+
+#ifdef __linux__
+  pthread_mutexattr_setprotocol(lv2_urid_manager->obj_mutexattr,
+				PTHREAD_PRIO_INHERIT);
+#endif
+
+  lv2_urid_manager->obj_mutex = (pthread_mutex_t *) malloc(sizeof(pthread_mutex_t));
+  pthread_mutex_init(lv2_urid_manager->obj_mutex,
+		     lv2_urid_manager->obj_mutexattr);
+
   lv2_urid_manager->id_counter = 1;
 
   lv2_urid_manager->urid = g_hash_table_new_full(g_str_hash, g_str_equal,
@@ -120,18 +115,6 @@ ags_lv2_urid_manager_init(AgsLv2UridManager *lv2_urid_manager)
 						 (GDestroyNotify) ags_lv2_urid_manager_destroy_data);
 }
 
-void
-ags_lv2_urid_manager_connect(AgsConnectable *connectable)
-{
-  /* empty */
-}
-
-void
-ags_lv2_urid_manager_disconnect(AgsConnectable *connectable)
-{
-  /* empty */
-}
-
 void
 ags_lv2_urid_manager_finalize(GObject *gobject)
 {
@@ -149,6 +132,21 @@ ags_lv2_urid_manager_finalize(GObject *gobject)
   G_OBJECT_CLASS(ags_lv2_urid_manager_parent_class)->finalize(gobject);
 }
 
+/**
+ * ags_lv2_urid_manager_get_class_mutex:
+ * 
+ * Get class mutex.
+ * 
+ * Returns: the class mutex of #AgsLv2UridManager
+ * 
+ * Since: 2.0.0
+ */
+pthread_mutex_t*
+ags_lv2_urid_manager_get_class_mutex()
+{
+  return(&ags_lv2_urid_manager_class_mutex);
+}
+
 void
 ags_lv2_urid_manager_destroy_data(gpointer data)
 {
@@ -175,18 +173,30 @@ ags_lv2_urid_manager_finder(gpointer key, gpointer value, gpointer user_data)
  * 
  * Returns: %TRUE on success, otherwise %FALSE
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 gboolean
 ags_lv2_urid_manager_insert(AgsLv2UridManager *lv2_urid_manager,
 			    gchar *uri, GValue *id)
 {
+  pthread_mutex_t *lv2_urid_manager_mutex;
+
   if(lv2_urid_manager == NULL ||
      uri == NULL ||
      id == NULL){
     return(FALSE);
   }
 
+  /* get lv2 uri map manager mutex */
+  pthread_mutex_lock(ags_lv2_urid_manager_get_class_mutex());
+  
+  lv2_urid_manager_mutex = lv2_urid_manager->obj_mutex;
+  
+  pthread_mutex_unlock(ags_lv2_urid_manager_get_class_mutex());
+
+  /*  */
+  pthread_mutex_lock(lv2_urid_manager_mutex);
+
   g_hash_table_insert(lv2_urid_manager->urid,
 		      uri, id);
 
@@ -202,7 +212,7 @@ ags_lv2_urid_manager_insert(AgsLv2UridManager *lv2_urid_manager,
  *
  * Returns: %TRUE as successfully removed, otherwise %FALSE
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 gboolean
 ags_lv2_urid_manager_remove(AgsLv2UridManager *lv2_urid_manager,
@@ -210,6 +220,18 @@ ags_lv2_urid_manager_remove(AgsLv2UridManager *lv2_urid_manager,
 {
   GValue *id;
 
+  pthread_mutex_t *lv2_urid_manager_mutex;
+
+  /* get lv2 uri map manager mutex */
+  pthread_mutex_lock(ags_lv2_urid_manager_get_class_mutex());
+  
+  lv2_urid_manager_mutex = lv2_urid_manager->obj_mutex;
+  
+  pthread_mutex_unlock(ags_lv2_urid_manager_get_class_mutex());
+
+  /*  */
+  pthread_mutex_lock(lv2_urid_manager_mutex);
+
   id = g_hash_table_lookup(lv2_urid_manager->urid,
 			   uri);
 
@@ -217,6 +239,8 @@ ags_lv2_urid_manager_remove(AgsLv2UridManager *lv2_urid_manager,
     g_hash_table_remove(lv2_urid_manager->urid,
 			uri);
   }
+
+  pthread_mutex_unlock(lv2_urid_manager_mutex);
   
   return(TRUE);
 }
@@ -230,7 +254,7 @@ ags_lv2_urid_manager_remove(AgsLv2UridManager *lv2_urid_manager,
  *
  * Returns: the id on success, else the newly created id
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 GValue*
 ags_lv2_urid_manager_lookup(AgsLv2UridManager *lv2_urid_manager,
@@ -238,6 +262,18 @@ ags_lv2_urid_manager_lookup(AgsLv2UridManager *lv2_urid_manager,
 {
   GValue *value;
 
+  pthread_mutex_t *lv2_urid_manager_mutex;
+
+  /* get lv2 uri map manager mutex */
+  pthread_mutex_lock(ags_lv2_urid_manager_get_class_mutex());
+  
+  lv2_urid_manager_mutex = lv2_urid_manager->obj_mutex;
+  
+  pthread_mutex_unlock(ags_lv2_urid_manager_get_class_mutex());
+
+  /*  */
+  pthread_mutex_lock(lv2_urid_manager_mutex);
+
   value = (GValue *) g_hash_table_lookup(lv2_urid_manager->urid,
 					 uri);
 
@@ -258,6 +294,8 @@ ags_lv2_urid_manager_lookup(AgsLv2UridManager *lv2_urid_manager,
     lv2_urid_manager->id_counter++;
   }
   
+  pthread_mutex_unlock(lv2_urid_manager_mutex);
+    
   return(value);
 }
 
@@ -348,7 +386,7 @@ ags_lv2_urid_manager_unmap(LV2_URID_Map_Handle handle,
  *
  * Returns: an instance of #AgsLv2UridManager
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsLv2UridManager*
 ags_lv2_urid_manager_get_instance()
@@ -375,7 +413,7 @@ ags_lv2_urid_manager_get_instance()
  *
  * Returns: a new #AgsLv2UridManager
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsLv2UridManager*
 ags_lv2_urid_manager_new()
